/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.common.mongodb;

import java.lang.reflect.ParameterizedType;

import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.data.mongodb.UncategorizedMongoDbException;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.index.IndexDefinition;
import org.springframework.data.mongodb.core.index.IndexOperations;
import org.springframework.data.mongodb.core.index.MongoPersistentEntityIndexResolver;

import com.mongodb.MongoCommandException;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public abstract class IndexCreator<T> {

	private final MongoTemplate mongoTemplate;

	@EventListener(ContextRefreshedEvent.class)
	public void createIndexes() {
		var mappingContext = mongoTemplate.getConverter().getMappingContext();

		var indexResolver = new MongoPersistentEntityIndexResolver(mappingContext);

		var indexOps = mongoTemplate.indexOps(getDocumentClass());
		indexResolver.resolveIndexFor(getDocumentClass()).forEach(indexDefinition -> ensureIndex(indexOps, indexDefinition));
	}

	private void ensureIndex(IndexOperations indexOps, IndexDefinition indexDefinition) {
		try {
			indexOps.ensureIndex(indexDefinition);
		} catch (UncategorizedMongoDbException e) {
			if (isNotIndexOptionsConflict(e)) {
				throw e;
			}
			recreateIndex(indexOps, indexDefinition);
		}
	}

	private boolean isNotIndexOptionsConflict(UncategorizedMongoDbException e) {
		return !(e.getCause() instanceof MongoCommandException) || ((MongoCommandException) e.getCause()).getCode() != 85;
	}

	private void recreateIndex(IndexOperations indexOps, IndexDefinition indexDefinition) {
		indexOps.dropIndex((String) indexDefinition.getIndexOptions().get("name"));
		indexOps.ensureIndex(indexDefinition);
	}

	@SuppressWarnings("unchecked")
	private Class<T> getDocumentClass() {
		return (Class<T>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0];
	}
}
