/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.common.mongodb;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.bson.BsonDocument;
import org.bson.Document;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedConstruction;
import org.springframework.data.mapping.context.MappingContext;
import org.springframework.data.mongodb.UncategorizedMongoDbException;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.convert.MongoConverter;
import org.springframework.data.mongodb.core.index.IndexOperations;
import org.springframework.data.mongodb.core.index.MongoPersistentEntityIndexResolver;
import org.springframework.data.mongodb.core.index.TextIndexDefinition;
import org.springframework.data.mongodb.core.mapping.MongoPersistentEntity;
import org.springframework.data.mongodb.core.mapping.MongoPersistentProperty;
import org.testcontainers.shaded.org.apache.commons.lang3.StringUtils;

import com.mongodb.MongoCommandException;
import com.mongodb.ServerAddress;
import com.thedeanda.lorem.LoremIpsum;

public abstract class AbstractIndexCreatorTest<C extends IndexCreator<D>, D> {

	@Mock
	protected MongoTemplate mongoTemplate;

	@Nested
	class TestInitIndexAfterStartUp {
		@Mock
		private MongoConverter converter;
		@Mock
		private MappingContext<? extends MongoPersistentEntity<?>, MongoPersistentProperty> mappingContext;

		private MockedConstruction<MongoPersistentEntityIndexResolver> mockedConstructionIndexResolver;

		@Mock
		private TextIndexDefinition indexDefinition;

		@Mock
		private IndexOperations indexOperations;

		private MappingContext<? extends MongoPersistentEntity<?>, MongoPersistentProperty> mappingContextUsedForConstructor;

		@BeforeEach
		@SuppressWarnings("unchecked")
		void setUpMocks() {
			when(mongoTemplate.getConverter()).thenReturn(converter);
			mockedConstructionIndexResolver = mockConstruction(MongoPersistentEntityIndexResolver.class, (mock, context) -> {
				doReturn(List.of(indexDefinition)).when(mock).resolveIndexFor(getDocumentClass());
				mappingContextUsedForConstructor = (MappingContext<? extends MongoPersistentEntity<?>, MongoPersistentProperty>) context.arguments()
						.get(0);
			});

			when(mongoTemplate.indexOps(getDocumentClass())).thenReturn(indexOperations);

		}

		@AfterEach
		void cleanUp() {
			mockedConstructionIndexResolver.close();
		}

		@Test
		void shouldGetMappingContext() {
			getIndexCreator().createIndexes();

			verify(converter).getMappingContext();
		}

		@Test
		void shouldCreateIndexResolver() {
			doReturn(mappingContext).when(converter).getMappingContext();

			getIndexCreator().createIndexes();

			assertThat(mockedConstructionIndexResolver.constructed()).hasSize(1);
		}

		@Test
		void shouldCreateIndexResolverWithMappingContext() {
			doReturn(mappingContext).when(converter).getMappingContext();

			getIndexCreator().createIndexes();

			assertThat(mappingContextUsedForConstructor).isEqualTo(mappingContext);
		}

		@Test
		void shouldGetIndexOperations() {
			getIndexCreator().createIndexes();

			verify(mongoTemplate).indexOps(getDocumentClass());
		}

		@Test
		void shouldResolveIndexForDocument() {
			getIndexCreator().createIndexes();

			verify(mockedConstructionIndexResolver.constructed().get(0)).resolveIndexFor(getDocumentClass());
		}

		@Test
		void shouldEnsureIndexForDocument() {
			getIndexCreator().createIndexes();

			verify(indexOperations).ensureIndex(indexDefinition);
		}

		@Nested
		class OnIndexOptionsConflict {

			private final String indexName = LoremIpsum.getInstance().getWords(1);
			private boolean dropped = false;

			private final BsonDocument response = BsonDocument
					.parse("{\"ok\": 0.0, \"errmsg\": \"\", \"code\": 85, \"codeName\": \"IndexOptionsConflict\"}");
			private final Document indexOptions = Document.parse(String.format(
					"{ v: 2, key: { _fts: 'text', _ftsx: 1 }, name: \"%s\", default_language: \"german\" }",
					indexName));

			@BeforeEach
			void setUpMocks() {
				when(indexDefinition.getIndexOptions()).thenReturn(indexOptions);
				doAnswer(invovation -> {
					if (dropped) {
						return StringUtils.EMPTY;
					} else {
						throw new UncategorizedMongoDbException(StringUtils.EMPTY, new MongoCommandException(response, new ServerAddress()));
					}
				}).when(indexOperations).ensureIndex(indexDefinition);
				doAnswer(invocation -> dropped = true).when(indexOperations).dropIndex(indexName);
			}

			@Test
			void shouldDropIndex() {
				getIndexCreator().createIndexes();

				verify(indexOperations).dropIndex(indexName);
			}

			@Test
			void shouldEnsureIndexTwice() {
				getIndexCreator().createIndexes();

				verify(indexOperations, times(2)).ensureIndex(indexDefinition);
			}
		}

		@Nested
		class OnDifferentMongoCommandExceptionCode {

			private final BsonDocument response = BsonDocument
					.parse("{\"ok\": 0.0, \"errmsg\": \"\", \"code\": 84, \"codeName\": \"IndexOptionsConflict\"}");

			@BeforeEach
			void setUpMocks() {
				when(indexOperations.ensureIndex(indexDefinition))
						.thenThrow(new UncategorizedMongoDbException(StringUtils.EMPTY, new MongoCommandException(response, new ServerAddress())));
			}

			@Test
			void shouldThrowException() {
				var indexCreator = getIndexCreator();

				assertThrows(UncategorizedMongoDbException.class, () -> indexCreator.createIndexes());
			}
		}

		@Nested
		class OnDifferentCause {

			@BeforeEach
			void setUpMocks() {
				when(indexOperations.ensureIndex(indexDefinition))
						.thenThrow(new UncategorizedMongoDbException(StringUtils.EMPTY, new RuntimeException()));
			}

			@Test
			void shouldThrowException() {
				var indexCreator = getIndexCreator();

				assertThrows(UncategorizedMongoDbException.class, () -> indexCreator.createIndexes());
			}
		}
	}

	protected abstract C getIndexCreator();

	protected abstract Class<D> getDocumentClass();
}
