/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.fachstelle;

import static de.ozgcloud.zufi.common.mongodb.DocumentTestUtils.*;
import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.autoconfigure.data.mongo.DataMongoTest;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.TextCriteria;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.common.test.DbInitializer;
import de.ozgcloud.zufi.common.mongodb.DocumentTestUtils;

@DataMongoTest
@ContextConfiguration(initializers = { DbInitializer.class }, classes = { FachstelleRepository.class, MongoOperations.class,
		FachstelleIndexCreator.class, DocumentTestUtils.class })
@ActiveProfiles({ "itcase", "with_db" })
@EnableAutoConfiguration
class FachstelleRepositoryITCase {

	@Autowired
	private FachstelleRepository fachstelleRepository;
	@Autowired
	private MongoOperations mongoOperations;

	@BeforeEach
	void cleanDb() {
		deleteAllFromCollection(Fachstelle.class);
	}

	@Nested
	class TestFindByMukId {

		@Test
		void shouldReturnEmpty() {
			var fachstelle = fachstelleRepository.findByMukId(FachstelleTestFactory.MUK_ID);

			assertThat(fachstelle).isEmpty();
		}

		@Test
		void shouldReturnFachstelle() {
			mongoOperations.save(FachstelleTestFactory.createWithoutId());

			var fachstelle = fachstelleRepository.findByMukId(FachstelleTestFactory.MUK_ID);

			assertThat(fachstelle).isPresent().get().usingRecursiveComparison(COMPARE_ALL_FIELDS_BUT_ID)
					.isEqualTo(FachstelleTestFactory.createWithoutId());
		}
	}

	@Nested
	class TestFindAllByScoreDesc {

		@Nested
		class TestFullTextSearchCapabilities {

			@BeforeEach
			void initData() {
				mongoOperations.save(Fachstelle.builder()
						.mukId("test-1")
						.registerNummer("reg13")
						.firmenName("Notare am Mühlenberg")
						.anschrift("Am Mühlenberg 1, 25551 Hohenlockstedt")
						.build());
				mongoOperations.save(Fachstelle.builder()
						.mukId("test-2")
						.registerNummer("reg42")
						.firmenName("TSV Lola")
						.anschrift("Große Freude 52, 25551 Hohenlockstedt")
						.build());
			}

			@Test
			void shouldBeCaseInsensitive() {
				var searchResult = callRepository("freude");

				assertThat(searchResult).hasSize(1).first().extracting(Fachstelle::getMukId).isEqualTo("test-2");
			}

			@Test
			void shouldMatchStemmedWords() {
				var searchResult = callRepository("notar");

				assertThat(searchResult).hasSize(1).extracting(Fachstelle::getMukId).contains("test-1");
			}

			@Test
			void shouldMatchAnyWordsInSearchBy() {
				var searchResult = callRepository("reg13 reg42");

				assertThat(searchResult).hasSize(2).extracting(Fachstelle::getMukId).contains("test-1", "test-2");
			}

			@Test
			void shouldStripPunctuationMarks() {
				var searchResult = callRepository("reg13;reg42");

				assertThat(searchResult).hasSize(2).extracting(Fachstelle::getMukId).contains("test-1", "test-2");
			}

			@Test
			void shouldFindAllMatchingFachstellen() {
				var searchResult = callRepository("Hohenlockstedt");

				assertThat(searchResult).hasSize(2).extracting(Fachstelle::getMukId).contains("test-1", "test-2");
			}

			@Test
			void shouldReturnEmptyListInCaseOfNoMatches() {
				var searchResult = callRepository("Berlin");

				assertThat(searchResult).isEmpty();
			}
		}

		@Nested
		class TestCompleteness {

			private static final String SEARCH_BY = LoremIpsum.getInstance().getWords(1);

			@Test
			void shouldSearchForMukId() {
				mongoOperations.save(Fachstelle.builder().mukId(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForFirmenName() {
				mongoOperations.save(Fachstelle.builder().firmenName(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForRechtsform() {
				mongoOperations.save(Fachstelle.builder().rechtsform(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForRechtsformText() {
				mongoOperations.save(Fachstelle.builder().rechtsformText(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForRegisterNummer() {
				mongoOperations.save(Fachstelle.builder().registerNummer(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForRegisterArt() {
				mongoOperations.save(Fachstelle.builder().registerArt(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForEmailAdresse() {
				mongoOperations.save(Fachstelle.builder().emailAdresse(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			@Test
			void shouldSearchForAnschrift() {
				mongoOperations.save(Fachstelle.builder().anschrift(SEARCH_BY).build());

				var searchResult = callRepository(SEARCH_BY);

				assertContainsOneResult(searchResult);
			}

			private void assertContainsOneResult(List<Fachstelle> searchResult) {
				assertThat(searchResult).hasSize(1);
			}
		}

		@Nested
		class TestSorting {

			private final Fachstelle fachstelle1 = Fachstelle.builder()
					.mukId("test-1")
					.registerNummer("reg13")
					.firmenName("Notare am Mühlenberg")
					.anschrift("Am Mühlenberg 1, 25551 Hohenlockstedt")
					.build();
			private final Fachstelle fachstelle2 = Fachstelle.builder()
					.mukId("test-2")
					.registerNummer("reg42")
					.firmenName("Notare am Hamburg")
					.anschrift("Große Freude 52, 20095 Hamburg")
					.build();

			@BeforeEach
			void initData() {
				mongoOperations.save(fachstelle1);
				mongoOperations.save(fachstelle2);
			}

			@Test
			void shouldHaveFachstelleWithMostHitsFirst() {
				var fachstellen = callRepository("Notar Hamburg");

				assertThat(fachstellen).usingRecursiveComparison().ignoringFields("score").isEqualTo(List.of(fachstelle2, fachstelle1));
			}
		}

		private List<Fachstelle> callRepository(String searchBy) {
			return fachstelleRepository.findAllByOrderByScoreDesc(TextCriteria.forDefaultLanguage().matching(searchBy));
		}
	}
}
