/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;

import java.util.Optional;
import java.util.Set;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.autoconfigure.data.mongo.DataMongoTest;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.TextCriteria;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.common.test.DbInitializer;
import de.ozgcloud.zufi.common.mongodb.DocumentTestUtils;

@DataMongoTest
@ContextConfiguration(initializers = { DbInitializer.class }, classes = { OrganisationsEinheitRepository.class, MongoOperations.class,
		OrganisationsEinheitIndexCreator.class, DocumentTestUtils.class })
@ActiveProfiles({ "itcase", "with_db" })
@EnableAutoConfiguration
class OrganisationsEinheitRepositoryITCase {

	@Autowired
	private OrganisationsEinheitRepository repository;
	@Autowired
	private MongoOperations operations;
	@Autowired
	private OrganisationsEinheitIndexCreator textIndexCreator;

	@BeforeEach
	void clearDatabase() {
		operations.dropCollection(OrganisationsEinheit.class);
	}

	@Nested
	class TestFindFirstByXzufiIdId {

		@Test
		void shouldFindOrganisationsEinheit() {
			var persistedOrganisationsEinheit = OrganisationsEinheitTestFactory.create();
			operations.save(persistedOrganisationsEinheit);

			var organisationsEinheit = repository.findFirstByXzufiIdId(XzufiIdTestFactory.ID);

			assertThat(organisationsEinheit).get().usingRecursiveComparison().isEqualTo(persistedOrganisationsEinheit);
		}

		@Test
		void shouldNotFindOrganisationsEinheit() {
			operations.save(OrganisationsEinheitTestFactory.createBuilder()
					.xzufiId(XzufiIdTestFactory.createBuilder().id(LoremIpsum.getInstance().getName()).build())
					.build());

			var organisationsEinheit = repository.findFirstByXzufiIdId(XzufiIdTestFactory.ID);

			assertThat(organisationsEinheit).isEmpty();
		}
	}

	@Nested
	class TestFindFirstByXZufiId {

		@Test
		void shouldFindOrganisationsEinheit() {
			var persistedOrganisationsEinheit = OrganisationsEinheitTestFactory.create();
			operations.save(persistedOrganisationsEinheit);

			var organisationsEinheit = callRepository();

			assertThat(organisationsEinheit.get()).usingRecursiveComparison().isEqualTo(persistedOrganisationsEinheit);
		}

		@Test
		void shouldNotFindOrganisationsEinheitDueToWrongOrganisationsEinheitId() {
			operations.save(OrganisationsEinheitTestFactory.createBuilder()
					.xzufiId(XzufiIdTestFactory.createBuilder().id(LoremIpsum.getInstance().getName()).build())
					.build());

			var organisationsEinheit = callRepository();

			assertThat(organisationsEinheit).isEmpty();
		}

		@Test
		void shouldNotFindOrganisationsEinheitDueToWrongSchemeAgencyId() {
			operations.save(OrganisationsEinheitTestFactory.createBuilder()
					.xzufiId(XzufiIdTestFactory.createBuilder().schemeAgencyId(LoremIpsum.getInstance().getName()).build())
					.build());

			var organisationsEinheit = callRepository();

			assertThat(organisationsEinheit).isEmpty();
		}

		private Optional<OrganisationsEinheit> callRepository() {
			return repository.findFirstByXzufiId(XzufiIdTestFactory.create());
		}
	}

	@Nested
	class TestDeleteByOrganisationsEinheitIdAndSchemeAgencyId {

		@Test
		void shouldDeleteOrganisationsEinheit() {
			operations.save(OrganisationsEinheitTestFactory.create());

			repository.deleteByXzufiId(XzufiIdTestFactory.create());

			assertThat(repository.count()).isZero();
		}

		@Test
		void shouldNotDeleteOrganisationsEinheitDueToWrongOrganisationsEinheitId() {
			operations.save(OrganisationsEinheitTestFactory.create());

			repository.deleteByXzufiId(XzufiIdTestFactory.createBuilder().id(LoremIpsum.getInstance().getWords(1)).build());

			assertThat(repository.count()).isOne();
		}

		@Test
		void shouldNotDeleteOrganisationsEinheitDueToWrongSchemeAgencyId() {
			operations.save(OrganisationsEinheitTestFactory.create());

			repository.deleteByXzufiId(XzufiIdTestFactory.createBuilder().schemeAgencyId(LoremIpsum.getInstance().getWords(1)).build());

			assertThat(repository.count()).isOne();
		}
	}

	@Nested
	class TestFindAllByZustaendigkeitenGebietIdOrderByScoreDesc {

		private final OrganisationsEinheit organisationsEinheit1 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.name("Amt für Finanzen")
				.synonyme("Steuern")
				.anschrift(AnschriftTestFactory.createBuilder().ort("Kiel").build())
				.build();
		private final OrganisationsEinheit organisationsEinheit2 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.name("Amt für Finanzen")
				.synonyme("")
				.zustaendigkeiten(Set.of(ZustaendigkeitTestFactory.createBuilder().gebietId("010001").build()))
				.anschrift(AnschriftTestFactory.createBuilder().ort("Flensburg").build())
				.build();
		private final OrganisationsEinheit organisationsEinheit3 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.name("Amt für Finanzen")
				.synonyme("Steuern")
				.zustaendigkeiten(Set.of(ZustaendigkeitTestFactory.createBuilder().gebietId("010001").build()))
				.anschrift(AnschriftTestFactory.createBuilder().ort("Flensburg").build())
				.build();

		@BeforeEach
		void initDatabase() {
			repository.save(organisationsEinheit1);
			repository.save(organisationsEinheit2);
			repository.save(organisationsEinheit3);
		}

		@Test
		void shouldFindMatchingEntries() {
			textIndexCreator.createIndexes();

			var organisationsEinheiten = repository.findAllByZustaendigkeitenGebietIdOrderByScoreDesc("010001",
					TextCriteria.forDefaultLanguage().matching("Finanz"));

			assertThat(organisationsEinheiten).usingRecursiveFieldByFieldElementComparatorIgnoringFields("score")
					.containsExactlyInAnyOrder(organisationsEinheit2, organisationsEinheit3);
		}

		@Test
		void shouldSort() {
			textIndexCreator.createIndexes();

			var organisationsEinheiten = repository.findAllByZustaendigkeitenGebietIdOrderByScoreDesc("010001",
					TextCriteria.forDefaultLanguage().matching("Finanz Steuer"));

			assertThat(organisationsEinheiten).usingRecursiveFieldByFieldElementComparatorIgnoringFields("score")
					.containsExactly(organisationsEinheit3, organisationsEinheit2);
		}
	}

	@Nested
	class TestFindAllOrderByScoreDesc {

		private final OrganisationsEinheit organisationsEinheit1 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.name("Amt für Finanzen")
				.synonyme("Steuern")
				.anschrift(AnschriftTestFactory.createBuilder().ort("Kiel").build())
				.build();
		private final OrganisationsEinheit organisationsEinheit2 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.name("Amt für Finanzen")
				.synonyme("")
				.zustaendigkeiten(Set.of(ZustaendigkeitTestFactory.createBuilder().gebietId("010001").build()))
				.anschrift(AnschriftTestFactory.createBuilder().ort("Flensburg").build())
				.build();
		private final OrganisationsEinheit organisationsEinheit3 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.name("Amt für Finanzen")
				.synonyme("Steuern")
				.zustaendigkeiten(Set.of(ZustaendigkeitTestFactory.createBuilder().gebietId("010001").build()))
				.anschrift(AnschriftTestFactory.createBuilder().ort("Flensburg").build())
				.build();

		@BeforeEach
		void initDatabase() {
			repository.save(organisationsEinheit1);
			repository.save(organisationsEinheit2);
			repository.save(organisationsEinheit3);
		}

		@Test
		void shouldFindMatchingEntries() {
			textIndexCreator.createIndexes();

			var organisationsEinheiten = repository.findAllByOrderByScoreDesc(TextCriteria.forDefaultLanguage().matching("Finanz"));

			assertThat(organisationsEinheiten).usingRecursiveFieldByFieldElementComparatorIgnoringFields("score")
					.containsExactlyInAnyOrder(organisationsEinheit1, organisationsEinheit2, organisationsEinheit3);
		}

		@Test
		void shouldSort() {
			textIndexCreator.createIndexes();

			var organisationsEinheiten = repository.findAllByOrderByScoreDesc(TextCriteria.forDefaultLanguage().matching("Finanz Steuer Kiel"));

			assertThat(organisationsEinheiten).usingRecursiveFieldByFieldElementComparatorIgnoringFields("score")
					.containsExactly(organisationsEinheit1, organisationsEinheit3, organisationsEinheit2);
		}
	}

	@Nested
	class TestFindByOrganisationsEinheitId {

		public static final String ORGANISATIONS_EINHEIT_ID = OrganisationsEinheitTestFactory.XZUFI_ID.getId();

		private final OrganisationsEinheit organisationsEinheit1 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.xzufiId(XzufiId.builder().id(ORGANISATIONS_EINHEIT_ID).schemeAgencyId(UUID.randomUUID().toString()).build())
				.build();
		private final OrganisationsEinheit organisationsEinheit2 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.xzufiId(XzufiId.builder().id(UUID.randomUUID().toString()).schemeAgencyId(UUID.randomUUID().toString()).build())
				.build();
		private final OrganisationsEinheit organisationsEinheit3 = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.xzufiId(XzufiId.builder().id(ORGANISATIONS_EINHEIT_ID).schemeAgencyId(UUID.randomUUID().toString()).build())
				.build();

		@BeforeEach
		void initDatabase() {
			repository.save(organisationsEinheit1);
			repository.save(organisationsEinheit2);
			repository.save(organisationsEinheit3);
		}

		@Test
		void shouldFindAllOrganisationsEinheitenWithId() {
			var result = repository.findByOrganisationsEinheitId(ORGANISATIONS_EINHEIT_ID);

			assertThat(result).usingRecursiveFieldByFieldElementComparatorIgnoringFields("id").containsExactlyInAnyOrder(organisationsEinheit1, organisationsEinheit3);
		}
	}
}
