/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.data.mongodb.core.query.TextCriteria;

import com.thedeanda.lorem.LoremIpsum;

class OrganisationsEinheitRepositoryTest {

	@Spy
	private OrganisationsEinheitRepository repository;

	@Nested
	class TestFindAllContainingInGebiet {
		private final String text = LoremIpsum.getInstance().getWords(5);
		private final String gebietId = UUID.randomUUID().toString();

		@Mock
		private TextCriteria textCriteria;

		@Test
		void shouldBuildTextQuery() {
			callRepository();

			verify(repository).buildTextCriteria(text);
		}

		@Test
		void shouldFindAllByZustaendigkeitenGebietIdOrderByScoreDesc() {
			doReturn(textCriteria).when(repository).buildTextCriteria(text);

			callRepository();

			verify(repository).findAllByZustaendigkeitenGebietIdOrderByScoreDesc(gebietId, textCriteria);
		}

		@Test
		void shouldReturnFoundOrganisationsEinheiten() {
			doReturn(textCriteria).when(repository).buildTextCriteria(text);
			var organisationsEinheiten = List.of(OrganisationsEinheitTestFactory.create());
			doReturn(organisationsEinheiten).when(repository).findAllByZustaendigkeitenGebietIdOrderByScoreDesc(gebietId, textCriteria);

			var found = callRepository();

			assertThat(found).isEqualTo(organisationsEinheiten);
		}

		private List<OrganisationsEinheit> callRepository() {
			return repository.findAllContainingInGebiet(text, gebietId);
		}
	}

	@Nested
	class TestFindAllContaining {
		private final String text = LoremIpsum.getInstance().getWords(5);

		@Mock
		private TextCriteria textCriteria;

		@Test
		void shouldBuildTextQuery() {
			callRepository();

			verify(repository).buildTextCriteria(text);
		}

		@Test
		void shouldFindAllByZustaendigkeitenGebietIdOrderByScoreDesc() {
			doReturn(textCriteria).when(repository).buildTextCriteria(text);

			callRepository();

			verify(repository).findAllByOrderByScoreDesc(textCriteria);
		}

		@Test
		void shouldReturnFoundOrganisationsEinheiten() {
			doReturn(textCriteria).when(repository).buildTextCriteria(text);
			var organisationsEinheiten = List.of(OrganisationsEinheitTestFactory.create());
			doReturn(organisationsEinheiten).when(repository).findAllByOrderByScoreDesc(textCriteria);

			var found = callRepository();

			assertThat(found).isEqualTo(organisationsEinheiten);
		}

		private List<OrganisationsEinheit> callRepository() {
			return repository.findAllContaining(text);
		}
	}

	@Nested
	class TestBuildTextCriteria {
		private final String text = LoremIpsum.getInstance().getWords(5);

		@Test
		void shouldReturnTextCriteria() {
			var expectedtextCriteria = TextCriteria.forDefaultLanguage().matching(text);

			var textCriteria = repository.buildTextCriteria(text);

			assertThat(textCriteria).usingRecursiveComparison().isEqualTo(expectedtextCriteria);
		}
	}
}
