/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import java.util.Set;

import org.springframework.stereotype.Component;

import de.ozgcloud.common.logging.OzgCloudLogging;
import de.ozgcloud.zufi.metadocument.MetaDocumentService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component
@OzgCloudLogging
@Log4j2
@RequiredArgsConstructor
public class PvogProcessor {

	private final PvogService pvogService;

	private final MetaDocumentService metaDocumentService;

	private final Set<TransferOperationEntryProcessor> processors;

	private final TransferOperationMapper transferOperationMapper;

	public void process() {
		int startIndex = getStartIndex();
		logSyncStarted(startIndex);
		processFromIndex(startIndex);
		logSyncFinished();
	}

	int getStartIndex() {
		return metaDocumentService.findNaechsterIndex().orElse(0);
	}

	void logSyncStarted(int startIndex) {
		if (startIndex == 0) {
			LOG.info("Start full sync");
		} else {
			LOG.info("Start incremental sync with index {}", startIndex);
		}
	}

	void logSyncFinished() {
		LOG.info("Finished sync");
	}

	void processFromIndex(int startIndex) {
		var importState = new ImportState(startIndex);
		do {
			var antwortTransfernachricht = fetchVerwaltungsObjekteAt(importState.index());
			process(antwortTransfernachricht);
			importState = importState.nextState(antwortTransfernachricht);
		} while (!isFinished(importState));
	}

	AntwortTransfernachricht fetchVerwaltungsObjekteAt(int index) {
		LOG.info("Fetching further data using index {}", index);
		return pvogService.getVerwaltungsObjekteAt(index);
	}

	void process(AntwortTransfernachricht antwortTransfernachricht) {
		if (pvogService.containsVerwaltungsObjekte(antwortTransfernachricht)) {
			processVerwaltungsObjekteAndUpdateMetaData(antwortTransfernachricht);
		}
	}

	void processVerwaltungsObjekteAndUpdateMetaData(AntwortTransfernachricht antwortTransfernachricht) {
		process(toTransferOperation(antwortTransfernachricht));
		metaDocumentService.saveMetaDocument(antwortTransfernachricht);
	}

	private TransferOperation toTransferOperation(AntwortTransfernachricht transfernachricht) {
		return transferOperationMapper.fromAntwortTransfernachricht(transfernachricht);
	}

	private void process(TransferOperation transferOperation) {
		transferOperation.getEntries().forEach(entry -> processors.forEach(p -> p.process(entry)));
	}

	boolean isFinished(ImportState importState) {
		return !(pvogService.hasMoreData(importState.antwortTransfernachricht()) && importState.indexChanged());
	}

	record ImportState(int index, boolean indexChanged, AntwortTransfernachricht antwortTransfernachricht) {

		ImportState(int index) {
			this(index, true, null);
		}

		ImportState nextState(AntwortTransfernachricht antwortTransfernachricht) {
			return (antwortTransfernachricht.naechsterIndex() == index)
					? new ImportState(index, false, antwortTransfernachricht)
					: new ImportState(antwortTransfernachricht.naechsterIndex(), true, antwortTransfernachricht);
		}
	}
}
