/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import java.util.Objects;
import java.util.Set;

import org.springframework.stereotype.Component;

import de.ozgcloud.common.logging.OzgCloudLogging;
import lombok.RequiredArgsConstructor;

@Component
@OzgCloudLogging
@RequiredArgsConstructor
class ZustaendigkeitProcessor extends TransferOperationEntryProcessor {

	private final OrganisationsEinheitRepository repository;

	@Override
	void processWrite(TransferOperationEntry transferOperationEntry) {
		var zustaendigkeit = transferOperationEntry.getZustaendigkeit();
		var organisationsEinheitXzufiId = buildOrganisationsEinheitXzufiId(zustaendigkeit.getXzufiId());
		repository.findFirstByXzufiId(organisationsEinheitXzufiId).ifPresentOrElse(
				organisationsEinheit -> saveWithNewZustaendigkeit(zustaendigkeit, organisationsEinheit),
				() -> saveEmptyOrganisationsEinheit(zustaendigkeit, organisationsEinheitXzufiId));

	}

	private void saveWithNewZustaendigkeit(Zustaendigkeit zustaendigkeit, OrganisationsEinheit organisationsEinheit) {
		organisationsEinheit.getZustaendigkeiten().add(zustaendigkeit);
		repository.save(organisationsEinheit);
	}

	private OrganisationsEinheit saveEmptyOrganisationsEinheit(Zustaendigkeit zustaendigkeit, XzufiId organisationsEinheitXzufiId) {
		return repository.save(OrganisationsEinheit.builder()
				.xzufiId(organisationsEinheitXzufiId)
				.zustaendigkeiten(Set.of(zustaendigkeit))
				.build());
	}

	@Override
	void processDelete(TransferOperationEntry transferOperationEntry) {
		var zustaendigkeit = transferOperationEntry.getZustaendigkeit();
		var organisationsEinheitId = buildOrganisationsEinheitXzufiId(zustaendigkeit.getXzufiId());
		repository.findFirstByXzufiId(organisationsEinheitId)
				.ifPresent(organisationsEinheit -> removeZustaendigkeit(zustaendigkeit, organisationsEinheit));
	}

	XzufiId buildOrganisationsEinheitXzufiId(XzufiId zustaendigkeitXzufiId) {
		return XzufiId.builder()
				.id(extractOrganisationsEinheitId(zustaendigkeitXzufiId.getId()))
				.schemeAgencyId(zustaendigkeitXzufiId.getSchemeAgencyId())
				.build();
	}

	String extractOrganisationsEinheitId(String zustaendigkeitId) {
		return zustaendigkeitId.split("\\.")[1];
	}

	void removeZustaendigkeit(Zustaendigkeit zustaendigkeit, OrganisationsEinheit organisationsEinheit) {
		organisationsEinheit.getZustaendigkeiten().remove(zustaendigkeit);
		repository.save(organisationsEinheit);
	}

	@Override
	boolean shouldProcess(TransferOperationEntry transferOperationEntry) {
		return Objects.nonNull(transferOperationEntry.getZustaendigkeit());
	}

}
