/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.metadocument;

import static org.assertj.core.api.Assertions.*;

import java.util.Date;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.ContextConfiguration;

import de.ozgcloud.common.test.DbInitializer;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.zufi.common.Oauth2Initializer;
import de.ozgcloud.zufi.organisationseinheit.PvogProcessor;

@ITCase
@ContextConfiguration(initializers = { Oauth2Initializer.class, DbInitializer.class })
class MetaDocumentRepositoryITCase {
	@Autowired
	private MetaDocumentRepository metaDocumentRepository;

	@MockBean
	private PvogProcessor processor;

	@DisplayName("Test saving the MetaDocument")
	@Nested
	class TestSave {
		@DisplayName("insert a new MetaDocument")
		@Nested
		class InsertMetaDocument {
			@BeforeEach
			void clearEntries() {
				deleteEntries();
			}

			@Test
			void shouldInsert() {
				metaDocumentRepository.save(MetaDocumentTestFactory.create());

				var foundDocs = metaDocumentRepository.findAll();

				assertThat(foundDocs).hasSize(1);
			}

			@Test
			void shouldHaveOneEntry() {
				var doc = MetaDocumentTestFactory.create();
				metaDocumentRepository.save(doc);
				metaDocumentRepository.save(doc);

				var foundDocs = metaDocumentRepository.findAll();

				assertThat(foundDocs).hasSize(1);
			}
		}

		@DisplayName("update an existing MetaDocument")
		@Nested
		class UpdateMetaDocument {
			@Test
			void shouldUpdate() {
				var newDoc = metaDocumentRepository.save(MetaDocumentTestFactory.create());
				var now = new Date().getTime();
				var updatedDoc = metaDocumentRepository
						.save(newDoc.toBuilder().lastRead(now).lastReadComplete(false).build());

				assertThat(updatedDoc).usingRecursiveComparison().ignoringFields("lastRead",
						"lastReadComplete").isEqualTo(newDoc);
				assertThat(updatedDoc.getLastRead()).isEqualTo(now);
				assertThat(updatedDoc.isLastReadComplete()).isFalse();
			}
		}
	}

	@DisplayName("Test loading the MetaDocument")
	@Nested
	class TestGet {
		@DisplayName("Load the MetaDocument")
		@Nested
		class GetMetaDocument {
			private MetaDocument doc;

			@BeforeEach
			void initData() {
				deleteEntries();

				doc = metaDocumentRepository.save(MetaDocumentTestFactory.create());
			}

			@Test
			void shouldGet() {
				var foundDocs = metaDocumentRepository.findAll();

				assertThat(foundDocs.iterator().next()).usingRecursiveComparison().isEqualTo(doc);
			}
		}
	}

	void deleteEntries() {
		metaDocumentRepository.deleteAll();
	}

}
