/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.metadocument;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.zufi.organisationseinheit.AntwortTransfernachricht;
import de.ozgcloud.zufi.organisationseinheit.AntwortTransfernachrichtTestFactory;

class MetaDocumentServiceTest {
	@Spy
	@InjectMocks
	private MetaDocumentService service;

	@Mock
	private MetaDocumentRepository metaDocumentRepository;

	@DisplayName("Test saving of the MetaDocument")
	@Nested
	class TestSaveMetaDocument {

		private final AntwortTransfernachricht transfernachricht = AntwortTransfernachrichtTestFactory.create();

		@Test
		void shouldFindExistingDocument() {
			service.saveMetaDocument(transfernachricht);

			verify(service).find();
		}

		@Nested
		class TestInserting {
			private MockedStatic<MetaDocument> staticMetaDoc;
			@Mock
			private MetaDocument.MetaDocumentBuilder builder;
			private final MetaDocument metaDocument = MetaDocument.builder().build();

			@BeforeEach
			void setUpMock() {
				staticMetaDoc = mockStatic(MetaDocument.class);
				staticMetaDoc.when(MetaDocument::builder).thenReturn(builder);
				when(builder.build()).thenReturn(metaDocument);
				doReturn(metaDocument).when(service).updateMetaDocument(metaDocument, transfernachricht);
			}

			@AfterEach
			void tearDown() {
				staticMetaDoc.close();
			}

			@Test
			void shouldBuildMetaDocument() {
				doReturn(Optional.empty()).when(service).find();

				service.saveMetaDocument(transfernachricht);

				verify(builder).build();
			}

			@Test
			void shouldUpdateMetaDocument() {
				doReturn(Optional.empty()).when(service).find();

				service.saveMetaDocument(transfernachricht);

				verify(service).updateMetaDocument(metaDocument, transfernachricht);
			}

			@Test
			void shouldSaveNewDocument() {
				doReturn(Optional.empty()).when(service).find();

				service.saveMetaDocument(transfernachricht);

				verify(metaDocumentRepository).save(metaDocument);
			}
		}

		@Nested
		class TestUpdating {
			private final MetaDocument metaDocument = MetaDocumentTestFactory.create();
			private final MetaDocument metaDocumentModified = MetaDocumentTestFactory.create();

			@BeforeEach
			void setUpMock() {
				doReturn(metaDocumentModified).when(service).updateMetaDocument(metaDocument, transfernachricht);
			}

			@Test
			void shouldUpdateFoundDocument() {
				doReturn(Optional.of(metaDocument)).when(service).find();

				service.saveMetaDocument(transfernachricht);

				verify(service).updateMetaDocument(metaDocument, transfernachricht);
			}

			@Test
			void shouldSaveUpdatedDocument() {
				doReturn(Optional.of(metaDocument)).when(service).find();

				service.saveMetaDocument(transfernachricht);

				verify(service).save(metaDocumentModified);
			}
		}

	}

	@Nested
	class TestFind {
		private MetaDocument document = MetaDocumentTestFactory.create();

		@Test
		void shouldFindDocument() {
			service.find();

			verify(metaDocumentRepository).findAll();
		}

		@Test
		void shouldReturnDocument() {
			when(metaDocumentRepository.findAll()).thenReturn(List.of(document));

			var resultDocument = service.find();

			assertThat(resultDocument).hasValue(document);
		}

		@Test
		void shouldReturnEmpty() {
			when(metaDocumentRepository.findAll()).thenReturn(Collections.emptyList());

			var resultDocument = service.find();

			assertThat(resultDocument).isEmpty();
		}
	}

	@Nested
	class TestFindNaechsterIndex {
		private MetaDocument document = MetaDocumentTestFactory.create();

		@Test
		void shouldFindNaechsterIndex() {
			service.findNaechsterIndex();

			verify(service).find();
		}

		@Test
		void shouldReturnNaechsterIndex() {
			doReturn(Optional.of(document)).when(service).find();

			var result = service.findNaechsterIndex();

			assertThat(result).hasValue(document.getNaechsterIndex());
		}
	}

	@Nested
	class TestUpdateMetaDocument {
		@Test
		void shouldSetNaechsteAnfrageUrl() {
			var updated = updateMetaDocument();

			assertThat(updated.getNaechsteAnfrageUrl()).isEqualTo(AntwortTransfernachrichtTestFactory.NEXT_URL);
		}

		@Test
		void shouldSetNaechsterIndex() {
			var updated = updateMetaDocument();

			assertThat(updated.getNaechsterIndex()).isEqualTo(AntwortTransfernachrichtTestFactory.NEXT_INDEX);
		}

		@Test
		void shouldSetLastReadComplete() {
			var updated = updateMetaDocument();

			assertThat(updated.isLastReadComplete()).isEqualTo(AntwortTransfernachrichtTestFactory.VOLLSTAENDIG);
		}

		@Test
		void shouldSetLastRead() {
			var updated = updateMetaDocument();

			assertThat(updated.getLastRead()).isNotEqualTo(MetaDocumentTestFactory.LAST_READ);
		}

		@Test
		void shouldSetId() {
			var updated = service.updateMetaDocument(MetaDocumentTestFactory.createBuilder().id(null).build(),
					AntwortTransfernachrichtTestFactory.create());

			assertThat(updated.getId()).isNotNull();
		}

		private MetaDocument updateMetaDocument() {
			return service.updateMetaDocument(MetaDocumentTestFactory.create(),
					AntwortTransfernachrichtTestFactory.create());
		}
	}

	@Nested
	class TestSave {
		private MetaDocument document = MetaDocumentTestFactory.create();

		@Test
		void shouldCallRepository() {
			service.save(document);

			verify(metaDocumentRepository).save(document);
		}

		@Test
		void shouldReturnDocument() {
			when(metaDocumentRepository.save(document)).thenReturn(document);

			var savedDocument = service.save(document);

			assertThat(savedDocument).isEqualTo(document);
		}
	}
}
