/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;
import org.testcontainers.shaded.org.apache.commons.lang3.RandomStringUtils;

import de.ozgcloud.zufi.schema.AnschriftOrganisationseinheit;

class AnschriftMapperTest {

	@Spy
	private final AnschriftMapper mapper = Mappers.getMapper(AnschriftMapper.class);

	@Nested
	class TestFromXzufiAnschriften {

		@Nested
		class OnEmptyAnschriften {

			@Test
			void shouldReturnEmptyAnschrift() {
				var anschrift = mapper.fromXzufiAnschriften(Collections.emptyList());

				assertThat(anschrift).usingRecursiveComparison().isEqualTo(Anschrift.builder().build());
			}
		}

		@Nested
		class OnPresentAnschriften {

			private final AnschriftOrganisationseinheit firstAnschrift = AnschriftOrganisationseinheitTestFactory.create();
			private final AnschriftOrganisationseinheit secondAnschrift = AnschriftOrganisationseinheitTestFactory.create();

			@Test
			void shouldMapFirstAnschrift() {
				callMapper();

				verify(mapper).fromXzufiAnschrift(firstAnschrift);
			}

			@Test
			void shouldReturnMappedAnschrift() {
				var mappedAnschrift = AnschriftTestFactory.create();
				doReturn(mappedAnschrift).when(mapper).fromXzufiAnschrift(firstAnschrift);

				var anschrift = callMapper();

				assertThat(anschrift).isEqualTo(mappedAnschrift);
			}

			private Anschrift callMapper() {
				return mapper.fromXzufiAnschriften(List.of(firstAnschrift, secondAnschrift));
			}
		}
	}

	@Nested
	class TestFromXzufiAnschrift {

		@BeforeEach
		void mockMapHausnummer() {
			doReturn(AnschriftTestFactory.HAUSNUMMER).when(mapper).mapHausnummer(AnschriftOrganisationseinheitTestFactory.HAUSNUMMER);
		}

		@Test
		void shouldCallMapHausnummer() {
			callMapper();

			verify(mapper).mapHausnummer(AnschriftOrganisationseinheitTestFactory.HAUSNUMMER);
		}

		@Test
		void shouldMapAnschrift() {
			var anschrift = callMapper();

			assertThat(anschrift).usingRecursiveComparison().isEqualTo(AnschriftTestFactory.create());
		}

		private Anschrift callMapper() {
			return mapper.fromXzufiAnschrift(AnschriftOrganisationseinheitTestFactory.create());
		}
	}

	@Nested
	class TestMapHausnummer {

		@Test
		void shouldStripInput() {
			var hausnummer = RandomStringUtils.randomNumeric(4);

			var returnedHausnummer = mapper.mapHausnummer("   " + hausnummer + "    ");

			assertThat(returnedHausnummer).isEqualTo(hausnummer);
		}

		@Test
		void shouldHandleNullHausnummer() {
			var hausnummer = mapper.mapHausnummer(null);

			assertThat(hausnummer).isNull();
		}
	}
}
