/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class OrganisationsEinheitProcessorTest {
	@Spy
	@InjectMocks
	private OrganisationsEinheitProcessor processor;

	@Mock
	private OrganisationsEinheitRepository repository;

	@Nested
	class TestProcessWrite {
		private final TransferOperationEntry organisationsEinheitEntry = TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry();
		@Captor
		private ArgumentCaptor<OrganisationsEinheit> organisationsEinheitCaptor;

		@Test
		void shouldFindExistingOrganisationsEinheit() {
			callProcessor();

			verify(repository).findFirstByXzufiId(OrganisationsEinheitTestFactory.XZUFI_ID);
		}

		@Nested
		class OnExistingOrganisationsEinheit {
			private final OrganisationsEinheit persistedOrganisationsEinheit = OrganisationsEinheitTestFactory.create();

			@BeforeEach
			void mockRepository() {
				when(repository.findFirstByXzufiId(OrganisationsEinheitTestFactory.XZUFI_ID)).thenReturn(Optional.of(persistedOrganisationsEinheit));
			}

			@Test
			void shouldPreserveExistingData() {
				callProcessor();

				verify(processor).updateOrganisationsEinheit(persistedOrganisationsEinheit, TransferOperationEntryTestFactory.ORGANISATIONS_EINHEIT);
			}

			@Test
			void shouldSaveOrganisationsEinheitWithExistingData() {
				var organisationsEinheitToSave = OrganisationsEinheitTestFactory.create();
				doReturn(organisationsEinheitToSave).when(processor).updateOrganisationsEinheit(persistedOrganisationsEinheit,
						TransferOperationEntryTestFactory.ORGANISATIONS_EINHEIT);

				callProcessor();

				verify(repository).save(organisationsEinheitToSave);
			}
		}

		@Nested
		class OnNewOrganisationsEinheit {
			@BeforeEach
			void mockRepository() {
				when(repository.findFirstByXzufiId(OrganisationsEinheitTestFactory.XZUFI_ID))
						.thenReturn(Optional.empty());
			}

			@Test
			void shouldSaveOrganisationsEinheit() {
				callProcessor();

				verify(repository).save(TransferOperationEntryTestFactory.ORGANISATIONS_EINHEIT);
			}

		}

		private void callProcessor() {
			processor.processWrite(organisationsEinheitEntry);
		}
	}

	@Nested
	class TestProcessDelete {
		@Test
		void shouldCallRepository() {
			processor.processDelete(TransferOperationEntryTestFactory.createDeleteOrganisationsEinheitEntry());

			verify(repository).deleteByXzufiId(OrganisationsEinheitTestFactory.XZUFI_ID);
		}
	}

	@Nested
	class TestShouldProcess {
		@Test
		void shouldReturnTrue() {
			var returnValue = processor.shouldProcess(TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry());

			assertThat(returnValue).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var returnValue = processor.shouldProcess(TransferOperationEntryTestFactory.createWriteEntry());

			assertThat(returnValue).isFalse();
		}
	}

	@Nested
	class TestUpdateOrganisationsEinheit {
		private final Zustaendigkeit zustaendigkeit1 = ZustaendigkeitTestFactory.createWithId(UUID.randomUUID().toString());
		private final Zustaendigkeit zustaendigkeit2 = ZustaendigkeitTestFactory.createWithId(UUID.randomUUID().toString());
		private final OrganisationsEinheit persistedOrganisationsEinheit = OrganisationsEinheitTestFactory.createBuilder()
				.zustaendigkeiten(Set.of(zustaendigkeit1, zustaendigkeit2))
				.build();
		private final OrganisationsEinheit transferredOrganisationsEinheit = OrganisationsEinheitTestFactory.createBuilder()
				.id(null)
				.zustaendigkeiten(new HashSet<>())
				.name(LoremIpsum.getInstance().getWords(5))
				.vorgangManagerAddress(null)
				.build();
		private final OrganisationsEinheit mergedOrganisationsEinheit = OrganisationsEinheitTestFactory.createBuilder()
				.zustaendigkeiten(Set.of(zustaendigkeit1, zustaendigkeit2))
				.name(transferredOrganisationsEinheit.getName())
				.build();

		@Test
		void shouldReturnMergedOrganisationsEinheit() {
			var resultOrganisationsEinheit = processor.updateOrganisationsEinheit(persistedOrganisationsEinheit, transferredOrganisationsEinheit);

			assertThat(resultOrganisationsEinheit).usingRecursiveComparison().ignoringCollectionOrder().isEqualTo(mergedOrganisationsEinheit);
		}
	}

}
