/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static de.ozgcloud.zufi.common.mongodb.DocumentTestUtils.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.test.context.ContextConfiguration;

import de.ozgcloud.common.test.DbInitializer;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.zufi.common.Oauth2Initializer;
import de.ozgcloud.zufi.metadocument.MetaDocument;
import de.ozgcloud.zufi.schema.TransferOperation040502;

@ITCase
@ContextConfiguration(initializers = { Oauth2Initializer.class, DbInitializer.class })
class PvogProcessorITCase {
	@Autowired
	@InjectMocks
	private PvogProcessor processor;

	@SpyBean
	private PvogService pvogService;

	@Autowired
	private MongoOperations mongoOperations;

	@DisplayName("Test processing PVOG data")
	@Nested
	class TestPvogProcessor {

		@BeforeEach
		void prepareIndex() {
			deleteAllFromCollection(OrganisationsEinheit.class);
			deleteAllFromCollection(MetaDocument.class);
		}

		@DisplayName("Test inserting PVOG data")
		@Nested
		class TestPvogProcessorSchreibeObjekte {

			@BeforeEach
			void initRemoteService() {
				doReturn(AntwortTransfernachrichtTestFactory.create()).when(pvogService).getVerwaltungsObjekteAt(0);
				doReturn(AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE_INCREMENT)
						.build())
								.when(pvogService).getVerwaltungsObjekteAt(AntwortTransfernachrichtTestFactory.NEXT_INDEX);
			}

			@BeforeEach
			void init() {
				doReturn(true).when(pvogService).containsVerwaltungsObjekte(any());
			}

			@Test
			void shouldProcessFull() {
				processor.process();

				assertThat(getMetaDocumentReponse()).hasSize(1);
				assertThat(getOrganisationsEinheitResponse()).hasSize(57);
			}

			@Test
			void shouldProcessIncremental() {
				shouldProcessFull();

				processor.process();

				assertThat(getMetaDocumentReponse()).hasSize(1);
				assertThat(getOrganisationsEinheitResponse())
						.hasSize(AntwortTransfernachrichtTestFactory.ANZAHL_ORGANISATIONSEINHEITEN + 1);
			}
		}

		@DisplayName("Test deleting PVOG data")
		@Nested
		class TestPvogProcessorLoescheObjekte {
			private static final String KIEL_ID = "10324180";

			@BeforeEach
			void initData() {
				doReturn(AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte(createLoescheTransferObject()).build())
						.when(pvogService).getVerwaltungsObjekteAt(anyInt());
				doReturn(true).when(pvogService).containsVerwaltungsObjekte(any());

				processor.process();
			}

			private String createLoescheTransferObject() {
				var transferOperation = new TransferOperation040502();
				transferOperation.getTransferOperation().add(
						LoescheObjektTestFactory.createWithId(KIEL_ID));
				return XzufiTestUtils.marshal(transferOperation);
			}

			@Test
			void shouldProcess() {
				processor.process();

				assertThat(getMetaDocumentReponse()).hasSize(1);
				assertThat(getOrganisationsEinheitResponse())
						.noneMatch(content -> content.getXzufiId().getId().equals(KIEL_ID));
			}
		}

		Iterable<MetaDocument> getMetaDocumentReponse() {
			return mongoOperations.findAll(MetaDocument.class);
		}

		Iterable<OrganisationsEinheit> getOrganisationsEinheitResponse() {
			return mongoOperations.findAll(OrganisationsEinheit.class);
		}
	}
}
