/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Optional;
import java.util.Random;
import java.util.Set;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.zufi.metadocument.MetaDocumentService;
import de.ozgcloud.zufi.metadocument.MetaDocumentTestFactory;
import de.ozgcloud.zufi.organisationseinheit.PvogProcessor.ImportState;

class PvogProcessorTest {
	@Spy
	@InjectMocks
	private PvogProcessor processor;
	@Mock
	private PvogService service;
	@Mock
	private MetaDocumentService metaDocumentService;
	@Mock
	private Set<TransferOperationEntryProcessor> entryProcessors;
	@Mock
	private TransferOperationMapper transferOperationMapper;

	@Nested
	class TestProcess {

		private static final int INDEX = new Random().nextInt();

		@BeforeEach
		void init() {
			doReturn(INDEX).when(processor).getStartIndex();
			doNothing().when(processor).processFromIndex(INDEX);
		}

		@Test
		void shouldGetStartIndex() {
			processor.process();

			verify(processor).getStartIndex();
		}

		@Test
		void shouldProcessFromIndex() {
			processor.process();

			verify(processor).processFromIndex(INDEX);
		}

		@Test
		void shouldLogSyncStarted() {
			processor.process();

			verify(processor).logSyncStarted(INDEX);
		}

		@Test
		void shouldLogSyncFinished() {
			processor.process();

			verify(processor).logSyncFinished();
		}
	}

	@Nested
	class TestGetStartIndex {

		@Test
		void shouldFindNextIndex() {
			givenFindNaechsterIndexReturnsNextIndex();

			processor.getStartIndex();

			verify(metaDocumentService).findNaechsterIndex();
		}

		@Test
		void shouldReturnNextIndex() {
			givenFindNaechsterIndexReturnsNextIndex();

			var startIndex = processor.getStartIndex();

			assertThat(startIndex).isEqualTo(MetaDocumentTestFactory.NAECHSTER_INDEX);
		}

		@Test
		void shouldReturnZeroIfNextIndexNotFound() {
			givenFindNaechsterIndexReturnsEmpty();

			var startIndex = processor.getStartIndex();

			assertThat(startIndex).isZero();
		}

		private void givenFindNaechsterIndexReturnsNextIndex() {
			when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.of(MetaDocumentTestFactory.NAECHSTER_INDEX));
		}

		private void givenFindNaechsterIndexReturnsEmpty() {
			when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.empty());
		}
	}

	@Nested
	class TestProcessFromIndex {
		private static final int START_INDEX =  1000;
		private static final int IMPORT_STATE_1_INDEX = 2000;

		private final AntwortTransfernachricht antwortTransfernachricht1 = AntwortTransfernachrichtTestFactory.createBuilder().naechsterIndex(IMPORT_STATE_1_INDEX).build();
		private final ImportState importState1 = new ImportState(IMPORT_STATE_1_INDEX, true, antwortTransfernachricht1);

		@BeforeEach
		void mockProcessorMethods() {
			doReturn(antwortTransfernachricht1).when(processor).fetchVerwaltungsObjekteAt(START_INDEX);
			doNothing().when(processor).process(antwortTransfernachricht1);
		}

		@Nested
		class OnIsFinishedAfterProcessingFirstIndex {

			@BeforeEach
			void mockIsFinished() {
				doReturn(true).when(processor).isFinished(importState1);
			}

			@Test
			void shouldFetchVerwaltungsObjekteAtIndex() {
				processFromStartIndex();

				verify(processor).fetchVerwaltungsObjekteAt(START_INDEX);
			}

			@Test
			void shouldProcessAntwortTransfernachricht() {
				processFromStartIndex();

				verify(processor).process(antwortTransfernachricht1);
			}

			@Test
			void shouldCheckIfImportIsFinishedBasedOnNextState() {
				processFromStartIndex();

				verify(processor).isFinished(importState1);
			}
		}

		@Nested
		class OnIsNotFinishedAfterProcessingFirstIndex {
			private static final int IMPORT_STATE_2_INDEX = 3000;

			private final AntwortTransfernachricht antwortTransfernachricht2 = AntwortTransfernachrichtTestFactory.createBuilder().naechsterIndex(IMPORT_STATE_2_INDEX).build();
			private final ImportState importState2 = new ImportState(IMPORT_STATE_2_INDEX, true, antwortTransfernachricht2);

			@BeforeEach
			void mockProcessorMethods() {
				doReturn(antwortTransfernachricht2).when(processor).fetchVerwaltungsObjekteAt(IMPORT_STATE_1_INDEX);
				doNothing().when(processor).process(antwortTransfernachricht2);
			}

			@BeforeEach
			void mockIsFinished() {
				doReturn(false).when(processor).isFinished(importState1);
				doReturn(true).when(processor).isFinished(importState2);
			}

			@Test
			void shouldFetchVerwaltungsObjekteAtIndex() {
				processFromStartIndex();

				verify(processor).fetchVerwaltungsObjekteAt(IMPORT_STATE_1_INDEX);
			}

			@Test
			void shouldProcessAntwortTransfernachricht() {
				processFromStartIndex();

				verify(processor).process(antwortTransfernachricht2);
			}

			@Test
			void shouldCheckIfImportIsFinishedBasedOnNextState() {
				processFromStartIndex();

				verify(processor).isFinished(importState2);
			}
		}

		private void processFromStartIndex() {
			processor.processFromIndex(START_INDEX);
		}
	}

	@Nested
	class TestFetchVerwaltungsObjekteAtIndex {

		private static final int INDEX = new Random().nextInt();

		@Test
		void shouldCallPvogService() {
			processor.fetchVerwaltungsObjekteAt(INDEX);

			verify(service).getVerwaltungsObjekteAt(INDEX);
		}
	}

	@Nested
	class TestProcessAntwortTransfernachricht {

		private final AntwortTransfernachricht antwortTransfernachricht = AntwortTransfernachrichtTestFactory.create();

		@Test
		void shouldCheckForVerwaltungsObjekte() {
			processor.process(antwortTransfernachricht);

			verify(service).containsVerwaltungsObjekte(antwortTransfernachricht);
		}

		@Test
		void shouldDoNothingIfAntwortDoesNotContainVerwaltungsObjekte() {
			when(service.containsVerwaltungsObjekte(antwortTransfernachricht)).thenReturn(false);

			processor.process(antwortTransfernachricht);

			verify(processor, never()).processVerwaltungsObjekteAndUpdateMetaData(any());
		}

		@Test
		void shouldProcessVerwaltungsObjekte() {
			when(service.containsVerwaltungsObjekte(antwortTransfernachricht)).thenReturn(true);
			doNothing().when(processor).processVerwaltungsObjekteAndUpdateMetaData(antwortTransfernachricht);

			processor.process(antwortTransfernachricht);

			verify(processor).processVerwaltungsObjekteAndUpdateMetaData(antwortTransfernachricht);
		}
	}

	@Nested
	class TestProcessVerwaltungsObjekteAndUpdateMetaData {

		private final AntwortTransfernachricht antwortTransfernachricht = AntwortTransfernachrichtTestFactory.create();

		@Test
		void shouldProcessOneEntry() {
			var entry = TransferOperationEntryTestFactory.createWriteEntry();
			var transferOperation = TransferOperationTestFactory.createBuilder().entries(List.of(entry)).build();
			when(transferOperationMapper.fromAntwortTransfernachricht(antwortTransfernachricht)).thenReturn(transferOperation);

			processor.processVerwaltungsObjekteAndUpdateMetaData(antwortTransfernachricht);

			entryProcessors.forEach(entryProcessor -> verify(entryProcessor).process(entry));
		}

		@Test
		void shouldProcessMultipleEntries() {
			var entry1 = TransferOperationEntryTestFactory.createWriteEntry();
			var entry2 = TransferOperationEntryTestFactory.createWriteEntry();
			var entry3 = TransferOperationEntryTestFactory.createDeleteEntry();
			var transferOperation = TransferOperationTestFactory.createBuilder().entries(List.of(entry1, entry2, entry3)).build();
			when(transferOperationMapper.fromAntwortTransfernachricht(antwortTransfernachricht)).thenReturn(transferOperation);

			processor.processVerwaltungsObjekteAndUpdateMetaData(antwortTransfernachricht);

			entryProcessors.forEach(entryProcessor -> verify(entryProcessor).process(entry1));
			entryProcessors.forEach(entryProcessor -> verify(entryProcessor).process(entry2));
			entryProcessors.forEach(entryProcessor -> verify(entryProcessor).process(entry3));
		}

		@Test
		void shouldUpdateMetaData() {
			var transferOperation = TransferOperationTestFactory.create();
			when(transferOperationMapper.fromAntwortTransfernachricht(antwortTransfernachricht)).thenReturn(transferOperation);

			processor.processVerwaltungsObjekteAndUpdateMetaData(antwortTransfernachricht);

			verify(metaDocumentService).saveMetaDocument(antwortTransfernachricht);
		}
	}

	@Nested
	class TestIsFinished {
		private static final int DUMMY_INDEX = 100;

		private final AntwortTransfernachricht antwortTransfernachricht = AntwortTransfernachrichtTestFactory.create();

		@ParameterizedTest
		@ValueSource(booleans = {true, false})
		void shouldReturnTrueIfHasNoMoreData(boolean indexChanged) {
			var importState = new ImportState(DUMMY_INDEX, indexChanged, antwortTransfernachricht);
			when(service.hasMoreData(antwortTransfernachricht)).thenReturn(false);

			var finished = processor.isFinished(importState);

			assertThat(finished).isTrue();
		}

		@ParameterizedTest
		@ValueSource(booleans = {true, false})
		void shouldReturnTrueIfIndexNotChanged(boolean hasMoreData) {
			var importState = new ImportState(DUMMY_INDEX, false, antwortTransfernachricht);
			when(service.hasMoreData(antwortTransfernachricht)).thenReturn(hasMoreData);

			var finished = processor.isFinished(importState);

			assertThat(finished).isTrue();
		}

		@Test
		void shouldReturnFalseIfHasMoreDataAndIndexChanged() {
			var importState = new ImportState(DUMMY_INDEX, true, antwortTransfernachricht);
			when(service.hasMoreData(antwortTransfernachricht)).thenReturn(true);

			var finished = processor.isFinished(importState);

			assertThat(finished).isFalse();
		}
	}

	@Nested
	class TestImportState {

		@Nested
		class TestCustomConstructor {

			@Test
			void shouldSetIndexChangedToTrue() {
				var importState = new ImportState(100);

				assertThat(importState.indexChanged()).isTrue();
			}
		}

		@Nested
		class TestNextState {

			private final AntwortTransfernachricht antwortTransfernachricht = AntwortTransfernachrichtTestFactory.create();

			@Test
			void shouldHaveAntwortTransfernachricht() {
				var importState = new ImportState(100);

				var nextState = importState.nextState(antwortTransfernachricht);

				assertThat(nextState.antwortTransfernachricht()).isEqualTo(antwortTransfernachricht);
			}

			@Test
			void shouldHaveIndex() {
				var importState = new ImportState(100);

				var nextState = importState.nextState(antwortTransfernachricht);

				assertThat(nextState.index()).isEqualTo(AntwortTransfernachrichtTestFactory.NEXT_INDEX);
			}

			@Nested
			class OnNaechsterIndexEqualsIndex {

				@Test
				void shouldHaveIndexChangedEqualsFalse() {
					var importState = new ImportState(AntwortTransfernachrichtTestFactory.NEXT_INDEX);

					var nextState = importState.nextState(antwortTransfernachricht);

					assertThat(nextState.indexChanged()).isFalse();
				}
			}

			@Nested
			class OnNaechsterIndexDiffersFromIndex {

				@Test
				void shouldHaveIndexChangedEqualsTrue() {
					var importState = new ImportState(AntwortTransfernachrichtTestFactory.NEXT_INDEX - 100);

					var nextState = importState.nextState(antwortTransfernachricht);

					assertThat(nextState.indexChanged()).isTrue();
				}
			}
		}
	}
}
