/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Random;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class PvogServiceTest {

	@Spy
	@InjectMocks
	private PvogService service;
	@Mock
	private ZufiProperties zufiProperties;
	@Mock
	private PvogRemoteService remoteService;

	@Nested
	class TestGetIncrementVerwaltungsObjekte {
		private final int index = new Random().nextInt();
		private static final String ARS = LoremIpsum.getInstance().getWords(1);

		@BeforeEach
		void init() {
			when(zufiProperties.getAmtlicheRegionalSchluessel()).thenReturn(ARS);
		}

		@Test
		void shouldGetVerwaltungsObjekte() {
			callService();

			verify(remoteService).getVerwaltungsObjekte(index, ARS);
		}

		@Test
		void shouldReturnVerwaltungsObjekte() {
			var expected = AntwortTransfernachrichtTestFactory.create();
			when(remoteService.getVerwaltungsObjekte(index, ARS)).thenReturn(expected);

			var result = callService();

			assertThat(result).isEqualTo(expected);
		}

		private AntwortTransfernachricht callService() {
			return service.getVerwaltungsObjekteAt(index);
		}

	}

	@DisplayName("Test if there is more dat ato fetch from the remote service")
	@Nested
	class TestHasMoreData {

		@Test
		void shouldReturnTrue() {
			var antwortTransfernachricht = AntwortTransfernachrichtTestFactory.createBuilder().vollstaendig(false).build();

			var res = service.hasMoreData(antwortTransfernachricht);

			assertThat(res).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var antwortTransfernachricht = AntwortTransfernachrichtTestFactory.create();

			var res = service.hasMoreData(antwortTransfernachricht);

			assertThat(res).isFalse();
		}
	}

	@DisplayName("Verify the xzufi data of the incomming AntwortTransfernachricht")
	@Nested
	class TestDataAvailable {

		@Test
		void shouldHandleEmptyBody() {
			var emptyBody = AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte("").build();

			var result = service.containsVerwaltungsObjekte(emptyBody);

			assertThat(result).isFalse();
		}

		@Test
		void shouldHandleNullBody() {
			var nullBody = AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte(null).build();

			var result = service.containsVerwaltungsObjekte(nullBody);

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnTrue() {
			var validNachricht = AntwortTransfernachrichtTestFactory.create();

			var result = service.containsVerwaltungsObjekte(validNachricht);

			assertThat(result).isTrue();
		}
	}
}