/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.zufi.common.OperationType;
import de.ozgcloud.zufi.schema.LoescheObjekt;
import de.ozgcloud.zufi.schema.SchreibeObjekt;
import de.ozgcloud.zufi.schema.Zustaendigkeit;
import de.ozgcloud.zufi.schema.ZustaendigkeitOrganisationseinheit;
import de.ozgcloud.zufi.schema.ZustaendigkeitTransferObjekt;

class TransferOperationEntryMapperTest {
	@Spy
	@InjectMocks
	private final TransferOperationEntryMapper mapper = Mappers.getMapper(TransferOperationEntryMapper.class);
	@Mock
	private OrganisationsEinheitMapper organisationsEinheitMapper;
	@Mock
	private ZustaendigkeitMapper zustaendigkeitMapper;

	@Nested
	class TestMapEntry {
		@Test
		void shouldCallMapSchreibeObjekt() {
			var schreibeObjekt = SchreibeObjektTestFactory.createWithOrganisationseinheit();

			mapper.mapEntry(schreibeObjekt);

			verify(mapper).mapSchreibeObjekt(schreibeObjekt);
		}

		@Test
		void shouldCallMapLoescheObjekt() {
			var loescheObjekt = LoescheObjektTestFactory.create();

			mapper.mapEntry(loescheObjekt);

			verify(mapper).mapLoescheObjekt(loescheObjekt);
		}

		@Test
		void shouldSetOperationTypeUnknown() {
			var unknownObject = "";

			var operationEntry = mapper.mapEntry(unknownObject);

			assertThat(operationEntry.getOperationType()).isEqualTo(OperationType.UNKNOWN);
		}
	}

	@Nested
	class TestMapSchreibeObjekt {
		@Nested
		class OnOrganisationseinheitInSchreibeObjekt {
			private final SchreibeObjekt schreibeObjekt = SchreibeObjektTestFactory.createWithOrganisationseinheit();

			@Test
			void shouldCallOrganisationsEinheitMapper() {
				callMapper();

				verify(organisationsEinheitMapper).fromXZufiOrganisationseinheit(schreibeObjekt.getOrganisationseinheit());
			}

			@Test
			void shouldSetOperationType() {
				var operationType = callMapper();

				assertThat(operationType.getOperationType()).isEqualTo(OperationType.WRITE);
			}

			private TransferOperationEntry callMapper() {
				return mapper.mapSchreibeObjekt(schreibeObjekt);
			}
		}

		@Nested
		class OnZustaendigkeitInSchreibeObjekt {
			private final SchreibeObjekt schreibeObjekt = SchreibeObjektTestFactory.createWithZustaendigkeit();

			@Test
			void shouldCallZustaendigkeitMapper() {
				callMapper();

				verify(zustaendigkeitMapper).fromZustaendigkeitTransferObjekt(schreibeObjekt.getZustaendigkeitTransferObjekt());
			}

			@Test
			void shouldSetOperationType() {
				var operationType = callMapper();

				assertThat(operationType.getOperationType()).isEqualTo(OperationType.WRITE);
			}

			private TransferOperationEntry callMapper() {
				return mapper.mapSchreibeObjekt(schreibeObjekt);
			}
		}
	}

	@Nested
	class TestMapsToZustaendigkeitOrganisationsEinheit {
		@Test
		void shouldMap() {
			var shouldMap = mapper.mapsToZustaendigkeitOrganisationsEinheit(ZustaendigkeitTransferObjektTestFactory.create());

			assertThat(shouldMap).isTrue();
		}

		@Test
		void shouldNotMapMissingZustaendigkeiten() {
			var shouldMap = mapper.mapsToZustaendigkeitOrganisationsEinheit(new ZustaendigkeitTransferObjekt());

			assertThat(shouldMap).isFalse();
		}

		@Test
		void shouldNotMapOtherZustaendigkeiten() {
			var shouldMap = mapper.mapsToZustaendigkeitOrganisationsEinheit(createTransferObjektWithDifferentZustaendigkeit());

			assertThat(shouldMap).isFalse();
		}

		private ZustaendigkeitTransferObjekt createTransferObjektWithDifferentZustaendigkeit() {
			var transferObjekt = new ZustaendigkeitTransferObjekt();
			var zustaendigkeit = createZustaendigkeitOtherThanOrganisationsEinheit();
			transferObjekt.getZustaendigkeit().add(zustaendigkeit);
			return transferObjekt;
		}

		private Zustaendigkeit createZustaendigkeitOtherThanOrganisationsEinheit() {
			var zustaendigkeit = new ZustaendigkeitOrganisationseinheit();
			var id = IdentifikatorTestFactory.create(UUID.randomUUID().toString());
			id.setSchemeID(LoremIpsum.getInstance().getWords(2));
			zustaendigkeit.setId(id);
			return zustaendigkeit;
		}

	}

	@Nested
	class TestMapLoescheObjekt {

		@Nested
		class OnOrganisationseinheit {
			private final LoescheObjekt loescheObjekt = LoescheObjektTestFactory.createForOrganisationseinheit();

			@Test
			void shouldSetOrganisationsEinheitId() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getOrganisationsEinheit().getXzufiId().getId()).isEqualTo(loescheObjekt.getId().getValue());
			}

			@Test
			void shouldSetOrganisationsEinheitSchemeAgencyId() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getOrganisationsEinheit().getXzufiId().getSchemeAgencyId())
						.isEqualTo(loescheObjekt.getId().getSchemeAgencyID());
			}

			@Test
			void shouldNotSetZustaendigkeit() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getZustaendigkeit()).isNull();
			}

			@Test
			void shouldSetOperationType() {
				var operationType = callMapper();

				assertThat(operationType.getOperationType()).isEqualTo(OperationType.DELETE);
			}

			private TransferOperationEntry callMapper() {
				return mapper.mapLoescheObjekt(loescheObjekt);
			}
		}

		@Nested
		class OnZustaendigkeit {
			private final LoescheObjekt loescheObjekt = LoescheObjektTestFactory.createForZustaendigkeit();

			@Test
			void shouldNotSetOrganisationsEinheitId() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getOrganisationsEinheit()).isNull();
			}

			@Test
			void shouldSetZustaendigkeitId() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getZustaendigkeit().getXzufiId().getId()).isEqualTo(loescheObjekt.getId().getValue());
			}

			@Test
			void shouldSetZustaendigkeitSchemeAgencyId() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getZustaendigkeit().getXzufiId().getSchemeAgencyId()).isEqualTo(loescheObjekt.getId().getSchemeAgencyID());
			}

			@Test
			void shouldSetOperationType() {
				var operationType = callMapper();

				assertThat(operationType.getOperationType()).isEqualTo(OperationType.DELETE);
			}

			private TransferOperationEntry callMapper() {
				return mapper.mapLoescheObjekt(loescheObjekt);
			}
		}

		@Nested
		class OnUnknownClass {
			private final LoescheObjekt loescheObjekt = LoescheObjektTestFactory.create();

			@Test
			void shouldNotSetOrganisationsEinheitId() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getOrganisationsEinheit()).isNull();
			}

			@Test
			void shouldNotSetZustaendigkeit() {
				var operationEntry = callMapper();

				assertThat(operationEntry.getZustaendigkeit()).isNull();
			}

			@Test
			void shouldSetOperationType() {
				var operationType = callMapper();

				assertThat(operationType.getOperationType()).isEqualTo(OperationType.DELETE);
			}

			private TransferOperationEntry callMapper() {
				return mapper.mapLoescheObjekt(loescheObjekt);
			}
		}

	}

}
