/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import java.util.StringJoiner;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

class ZustaendigkeitProcessorTest {

	@Spy
	@InjectMocks
	private ZustaendigkeitProcessor processor;

	@Mock
	private OrganisationsEinheitRepository repository;

	@Nested
	class TestProcessWrite {

		private final TransferOperationEntry entry = TransferOperationEntryTestFactory.createWriteZustaendigkeitEntry();

		@Captor
		private ArgumentCaptor<OrganisationsEinheit> organisationsEinheitCaptor;

		private final XzufiId organisationsEinheitXzufiId = XzufiIdTestFactory.createWithId(UUID.randomUUID().toString());

		@BeforeEach
		void mockProcessor() {
			doReturn(organisationsEinheitXzufiId).when(processor).buildOrganisationsEinheitXzufiId(ZustaendigkeitTestFactory.XZUFI_ID);
		}

		@Test
		void shouldBuildOrganisationsEinheitXZufiId() {
			callProcessor();

			verify(processor).buildOrganisationsEinheitXzufiId(ZustaendigkeitTestFactory.XZUFI_ID);
		}

		@Test
		void shouldGetOrganisationsEinheit() {
			callProcessor();

			verify(repository).findFirstByXzufiId(organisationsEinheitXzufiId);
		}

		@Test
		void shouldSaveOrganisationsEinheit() {
			var organisationsEinheit = OrganisationsEinheitTestFactory.create();
			when(repository.findFirstByXzufiId(organisationsEinheitXzufiId)).thenReturn(Optional.of(organisationsEinheit));

			callProcessor();

			verify(repository).save(organisationsEinheit);
		}

		@Test
		void shouldSaveOrganisationsEinheitWithUpdatedZustaendigkeit() {
			var oldZustaendigkeit = ZustaendigkeitTestFactory.createWithId(UUID.randomUUID().toString());
			var organisationsEinheit = OrganisationsEinheitTestFactory.createBuilder()
					.zustaendigkeiten(new HashSet<>(Set.of(oldZustaendigkeit)))
					.build();
			when(repository.findFirstByXzufiId(organisationsEinheitXzufiId)).thenReturn(Optional.of(organisationsEinheit));

			callProcessor();

			verify(repository).save(organisationsEinheitCaptor.capture());
			assertThat(organisationsEinheitCaptor.getValue().getZustaendigkeiten()).containsExactlyInAnyOrder(oldZustaendigkeit,
					TransferOperationEntryTestFactory.ZUSTAENDIGKEIT);
		}

		@Test
		void shouldSaveNewOrganisationsEinheitIfMissing() {
			var expectedOrganisationsEinheit = OrganisationsEinheit.builder()
					.xzufiId(organisationsEinheitXzufiId)
					.zustaendigkeiten(Set.of(TransferOperationEntryTestFactory.ZUSTAENDIGKEIT))
					.build();
			when(repository.findFirstByXzufiId(organisationsEinheitXzufiId)).thenReturn(Optional.empty());

			callProcessor();

			verify(repository).save(organisationsEinheitCaptor.capture());
			assertThat(organisationsEinheitCaptor.getValue()).usingRecursiveComparison().isEqualTo(expectedOrganisationsEinheit);
		}

		private void callProcessor() {
			processor.processWrite(entry);
		}
	}

	@Nested
	class TestProcessDelete {

		private final TransferOperationEntry entry = TransferOperationEntryTestFactory.createDeleteZustaendigkeitEntry();

		@Captor
		private ArgumentCaptor<OrganisationsEinheit> organisationsEinheitCaptor;

		private final XzufiId organisationsEinheitXzufiId = XzufiIdTestFactory.createWithId(UUID.randomUUID().toString());

		@BeforeEach
		void mockProcessor() {
			doReturn(organisationsEinheitXzufiId).when(processor).buildOrganisationsEinheitXzufiId(ZustaendigkeitTestFactory.XZUFI_ID);
		}

		@Test
		void shouldBuildOrganisationsEinheitXZufiId() {
			callProcessor();

			verify(processor).buildOrganisationsEinheitXzufiId(ZustaendigkeitTestFactory.XZUFI_ID);
		}

		@Test
		void shouldFindFirstOrganisationsEinheit() {
			callProcessor();

			verify(repository).findFirstByXzufiId(organisationsEinheitXzufiId);
		}

		@Test
		void shouldRemoveZustaendigkeitIfFound() {
			var organisationsEinheit = OrganisationsEinheitTestFactory.create();
			when(repository.findFirstByXzufiId(organisationsEinheitXzufiId)).thenReturn(Optional.of(organisationsEinheit));

			callProcessor();

			verify(processor).removeZustaendigkeit(TransferOperationEntryTestFactory.ZUSTAENDIGKEIT, organisationsEinheit);
		}

		private void callProcessor() {
			processor.processDelete(entry);
		}
	}

	@Nested
	class TestBuildOrganisationsEinheitXzufiId {

		private XzufiId zustaendigkeitXzufiId = XzufiIdTestFactory.create();
		private String organisationsEinheitId = UUID.randomUUID().toString();

		@BeforeEach
		void mockProcessor() {
			doReturn(organisationsEinheitId).when(processor).extractOrganisationsEinheitId(XzufiIdTestFactory.ID);
		}

		@Test
		void shouldExtractOrganisationsEinheitId() {
			callProcessor();

			verify(processor).extractOrganisationsEinheitId(XzufiIdTestFactory.ID);
		}

		@Test
		void shouldReturnOrganisationsEinheitXzufiId() {
			var expectedId = XzufiIdTestFactory.createBuilder().id(organisationsEinheitId).build();

			var organisationsEinheitXzufiId = callProcessor();

			assertThat(organisationsEinheitXzufiId).usingRecursiveComparison().isEqualTo(expectedId);
		}

		private XzufiId callProcessor() {
			return processor.buildOrganisationsEinheitXzufiId(zustaendigkeitXzufiId);
		}
	}

	@Nested
	class TestExtractOrganisationsEinheitId {

		private final String leistungId = UUID.randomUUID().toString();
		private final String organisationsEinheitId = UUID.randomUUID().toString();
		private final String gebietId = UUID.randomUUID().toString();
		private final String zustaendigkeitId = new StringJoiner(".").add(leistungId).add(organisationsEinheitId).add(gebietId).toString();

		@Test
		void shouldReturnOrganisationsEinheitId() {
			var resultId = processor.extractOrganisationsEinheitId(zustaendigkeitId);

			assertThat(resultId).isEqualTo(organisationsEinheitId);
		}
	}

	@Nested
	class TestRemoveZustaendigkeit {
		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.createBuilder()
				.zustaendigkeiten(new HashSet<>(Set.of(TransferOperationEntryTestFactory.ZUSTAENDIGKEIT)))
				.build();

		@Captor
		private ArgumentCaptor<OrganisationsEinheit> organisationsEinheitCaptor;

		@Test
		void shouldSaveOrganisationsEinheit() {
			callProcessor();

			verify(repository).save(organisationsEinheit);
		}

		@Test
		void shouldRemoveZustaendigkeit() {
			callProcessor();

			verify(repository).save(organisationsEinheitCaptor.capture());
			assertThat(organisationsEinheitCaptor.getValue().getZustaendigkeiten()).isEmpty();
		}

		private void callProcessor() {
			processor.removeZustaendigkeit(TransferOperationEntryTestFactory.ZUSTAENDIGKEIT, organisationsEinheit);
		}
	}

	@Nested
	class TestShouldProcess {

		@Test
		void shouldReturnTrue() {
			var shouldProcess = processor.shouldProcess(TransferOperationEntryTestFactory.createWriteZustaendigkeitEntry());

			assertThat(shouldProcess).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var returnValue = processor.shouldProcess(TransferOperationEntryTestFactory.createWriteEntry());

			assertThat(returnValue).isFalse();
		}
	}
}
