/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.common.errorhandling;

import java.util.UUID;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.TechnicalException;
import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import io.grpc.Status;
import io.grpc.StatusException;
import jakarta.validation.ConstraintViolationException;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.advice.GrpcAdvice;
import net.devh.boot.grpc.server.advice.GrpcExceptionHandler;

@Log4j2
@GrpcAdvice
public class GrpcExceptionHandlers {

	static final String KEY_EXCEPTION_ID = "EXCEPTION_ID";

	@GrpcExceptionHandler
	public StatusException handleResourceNotFoundException(ResourceNotFoundException e) {
		return new StatusException(buildNotFoundStatus(e), buildMetadata(e.getExceptionId()));
	}

	private Status buildNotFoundStatus(ResourceNotFoundException e) {
		return Status.NOT_FOUND.withDescription(e.getMessage()).withCause(e.getCause());
	}

	@GrpcExceptionHandler
	public StatusException handleTechnicalException(TechnicalException e) {
		return new StatusException(buildInternalStatus(e), buildMetadata(e.getExceptionId()));
	}

	@GrpcExceptionHandler
	public StatusException handleConstraintViolationException(ConstraintViolationException e) {
		return buildStatusExceptionForStatus(Status.INVALID_ARGUMENT, e, "Constraint violation error: ");
	}

	private Status buildInternalStatus(RuntimeException e) {
		return Status.INTERNAL.withDescription(e.getMessage()).withCause(e.getCause());
	}

	@GrpcExceptionHandler
	public StatusException handleRuntimeException(RuntimeException e) {
		return buildStatusExceptionForStatus(Status.INTERNAL, e, "Grpc internal server error: ");
	}

	@GrpcExceptionHandler
	public StatusException handleMissingRequiredGrpcArgumentException(MissingRequiredGrpcArgumentException e) {
		return buildStatusExceptionForStatus(Status.INVALID_ARGUMENT, e, "Grpc invalid argument: ");
	}

	private StatusException buildStatusExceptionForStatus(Status status, Exception e, String logMessage) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error(logMessage + messageWithExceptionId, e);
		return new StatusException(buildStatus(status, e, messageWithExceptionId), buildMetadata(exceptionId));
	}

	String createExceptionId() {
		return UUID.randomUUID().toString();
	}

	private Status buildStatus(Status status, Exception e, String messageWithExceptionId) {
		return status.withDescription(messageWithExceptionId).withCause(e.getCause());
	}

	private Metadata buildMetadata(String exceptionId) {
		var metadata = new Metadata();
		addExceptionId(metadata, exceptionId);
		return metadata;
	}

	private void addExceptionId(Metadata metadata, String value) {
		metadata.put(createKeyOf(KEY_EXCEPTION_ID), value);
	}

	private Key<String> createKeyOf(String key) {
		return Key.of(key, Metadata.ASCII_STRING_MARSHALLER);
	}

}