/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import java.util.List;

import de.ozgcloud.zufi.common.errorhandling.MissingRequiredGrpcArgumentException;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerGetAddressRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerGetAddressResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerRegistrationRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerRegistrationResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.VorgangManagerRegistrationServiceGrpc.VorgangManagerRegistrationServiceImplBase;
import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
@RequiredArgsConstructor
class VorgangManagerRegistrationGrpcService extends VorgangManagerRegistrationServiceImplBase {

	private final VorgangManagerRegistrationService vorgangManagerRegistrationService;
	private final GrpcXzufiIdMapper xzufiIdMapper;

	private static final String NOT_FOUND_MESSAGE = "Could not find the following OrganisationsEinheitIds: ";
	private static final String NO_ORGANISATIONS_ID_PROVIDED_MESSAGE = "No OrganisationsEinheitIds provided in request!";
	private static final String NO_VORGANG_MANAGER_ADDRESS_PROVIDED_MESSAGE = "No Vorgang-Manager address provided in request!";

	@Override
	public void register(GrpcVorgangManagerRegistrationRequest request,
			StreamObserver<GrpcVorgangManagerRegistrationResponse> responseObserver) {
		validateServiceAddress(request);

		var notFoundOrganisationsEinheitIds = registerAndReturnNotFoundIds(request);

		responseObserver.onNext(buildRegisterResponse(notFoundOrganisationsEinheitIds));
		responseObserver.onCompleted();
	}

	void validateServiceAddress(GrpcVorgangManagerRegistrationRequest request) {
		if (request.getServiceAddress().isEmpty()) {
			throw new MissingRequiredGrpcArgumentException(NO_VORGANG_MANAGER_ADDRESS_PROVIDED_MESSAGE);
		}
	}

	List<String> registerAndReturnNotFoundIds(GrpcVorgangManagerRegistrationRequest request) {
		if (!request.getXzufiIdsList().isEmpty()) {
			return registerWithXzufiIds(request);
		}
		if (!request.getOrganisationsEinheitIdsList().isEmpty()) {
			return registerWithOrganisationsEinheitIds(request);
		}
		throw new MissingRequiredGrpcArgumentException(NO_ORGANISATIONS_ID_PROVIDED_MESSAGE);
	}

	List<String> registerWithXzufiIds(GrpcVorgangManagerRegistrationRequest request) {
		return request.getXzufiIdsList().stream()
				.filter(id -> !vorgangManagerRegistrationService.registerVorgangManager(request.getServiceAddress(), xzufiIdMapper.fromGrpc(id)))
				.map(id -> id.getSchemeAgencyId() + "." + id.getId())
				.toList();
	}

	@Deprecated(since = "1.1.0", forRemoval = true)
	List<String> registerWithOrganisationsEinheitIds(GrpcVorgangManagerRegistrationRequest request) {
		return request.getOrganisationsEinheitIdsList().stream()
				.filter(id -> !vorgangManagerRegistrationService.registerVorgangManagerWithOrganisationsEinheit(request.getServiceAddress(),
						id))
				.toList();
	}

	GrpcVorgangManagerRegistrationResponse buildRegisterResponse(List<String> notFoundOrganisationsEinheitIds) {
		var successful = notFoundOrganisationsEinheitIds.isEmpty();
		var responseBuilder = GrpcVorgangManagerRegistrationResponse.newBuilder().setSuccess(successful);
		if (!successful) {
			responseBuilder.setMessage(NOT_FOUND_MESSAGE + String.join(", ", notFoundOrganisationsEinheitIds));
		}
		return responseBuilder.build();
	}

	@Override
	public void getVorgangManagerAddress(GrpcVorgangManagerGetAddressRequest request,
			StreamObserver<GrpcVorgangManagerGetAddressResponse> responseObserver) {

		vorgangManagerRegistrationService.getVorgangManagerAddress(request.getOrganisationsEinheitId()).ifPresentOrElse(
				result -> buildGetAddressResponse(responseObserver, result),
				() -> throwOrganisationsEinheitNotFoundException(request));
	}

	private void buildGetAddressResponse(StreamObserver<GrpcVorgangManagerGetAddressResponse> responseObserver, String address) {
		responseObserver.onNext(GrpcVorgangManagerGetAddressResponse.newBuilder()
				.setVorgangManagerAddress(address)
				.build());
		responseObserver.onCompleted();
	}

	private void throwOrganisationsEinheitNotFoundException(GrpcVorgangManagerGetAddressRequest request) {
		throw new OrganisationsEinheitNotFoundException(request.getOrganisationsEinheitId());
	}
}
