/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.common.errorhandling;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Set;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.common.errorhandling.TechnicalException;
import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import io.grpc.Status.Code;
import io.grpc.StatusException;
import jakarta.validation.ConstraintViolationException;

class GrpcExceptionHandlersTest {

	private static final String LOG_MESSAGE = LoremIpsum.getInstance().getWords(10);

	@Spy
	@InjectMocks
	private GrpcExceptionHandlers handler;

	@Nested
	class TestHandleTechnicalException {

		private final TechnicalException exception = new TechnicalException(LOG_MESSAGE, new RuntimeException());

		@Test
		void shouldHaveStatusCode() {
			var status = handleException().getStatus();

			assertThat(status.getCode()).isEqualTo(Code.INTERNAL);
		}

		@Test
		void shouldHaveDescription() {
			var status = handleException().getStatus();

			assertThat(status.getDescription()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveExceptionId() {
			var metaData = handleException().getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exception.getExceptionId());
		}

		private StatusException handleException() {
			return handler.handleTechnicalException(exception);
		}
	}

	@Nested
	class TestHandleConstraintViolationException {

		private final String exceptionId = LoremIpsum.getInstance().getWords(1);

		@BeforeEach
		void mockExceptionId() {
			doReturn(exceptionId).when(handler).createExceptionId();
		}

		@Test
		void shouldHaveStatusCode() {
			var status = handleException(createConstraintViolationException()).getStatus();

			assertThat(status.getCode()).isEqualTo(Code.INVALID_ARGUMENT);
		}

		@Test
		void shouldHaveDescription() {
			var status = handleException(createConstraintViolationException()).getStatus();

			assertThat(status.getDescription()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveExceptionId() {
			var exception = createConstraintViolationException();

			var metaData = handleException(exception).getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exceptionId);
		}

		private ConstraintViolationException createConstraintViolationException() {
			return new ConstraintViolationException(LOG_MESSAGE, Set.of());
		}

		private StatusException handleException(ConstraintViolationException exception) {
			return handler.handleConstraintViolationException(exception);
		}
	}

	@Nested
	class TestHandleResourceNotFoundException {
		private static final String RESOURCE_ID = UUID.randomUUID().toString();
		private static final String RESOURCE_NAME = LoremIpsum.getInstance().getWords(1);

		private final ResourceNotFoundException exception = new ResourceNotFoundException(RESOURCE_ID, RESOURCE_NAME,
				new RuntimeException());

		@Test
		void shouldHaveStatusCode() {
			var status = handleException().getStatus();

			assertThat(status.getCode()).isEqualTo(Code.NOT_FOUND);
		}

		@Test
		void shouldHaveDescription() {
			var status = handleException().getStatus();

			assertThat(status.getDescription()).isEqualTo(exception.getMessage());
		}

		@Test
		void shouldHaveExceptionId() {
			var metaData = handleException().getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exception.getExceptionId());
		}

		private StatusException handleException() {
			return handler.handleResourceNotFoundException(exception);
		}
	}

	@Nested
	class TestHandleRuntimeException {

		private final String exceptionId = UUID.randomUUID().toString();
		private final RuntimeException exception = new RuntimeException(LOG_MESSAGE);

		@BeforeEach
		void mockExceptionId() {
			doReturn(exceptionId).when(handler).createExceptionId();
		}

		@Test
		void shouldHaveStatusCode() {
			var status = handleException().getStatus();

			assertThat(status.getCode()).isEqualTo(Code.INTERNAL);
		}

		@Test
		void shouldHaveMessage() {
			var statusException = handleException();

			assertThat(statusException.getMessage()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveIdInMessage() {
			var statusException = handleException();

			assertThat(statusException.getMessage()).contains(exceptionId);
		}

		@Test
		void shouldHaveMessageInDescription() {
			var status = handleException().getStatus();

			assertThat(status.getDescription()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveIdInDescription() {
			var status = handleException().getStatus();

			assertThat(status.getDescription()).contains(exceptionId);
		}

		@Test
		void shouldHaveExceptionId() {
			var metaData = handleException().getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exceptionId);
		}

		private StatusException handleException() {
			return handler.handleRuntimeException(exception);
		}
	}

	@Nested
	class TestHandleMissingRequiredGrpcArgumentException {

		private final String exceptionId = UUID.randomUUID().toString();
		private final Exception cause = new Exception();
		private final MissingRequiredGrpcArgumentException exception = new MissingRequiredGrpcArgumentException(LOG_MESSAGE, cause);

		@BeforeEach
		void mockExceptionId() {
			doReturn(exceptionId).when(handler).createExceptionId();
		}

		@Test
		void shouldHaveStatusCode() {
			var status = handleException().getStatus();

			assertThat(status.getCode()).isEqualTo(Code.INVALID_ARGUMENT);
		}

		@Test
		void shouldHaveMessage() {
			var statusException = handleException();

			assertThat(statusException.getMessage()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveExceptionId() {
			var statusException = handleException();

			assertThat(statusException.getMessage()).contains(exceptionId);
		}

		@Test
		void shouldHaveCause() {
			var statusException = handleException();

			assertThat(statusException.getCause()).isEqualTo(cause);
		}

		@Test
		void shouldHaveMessageInDescription() {
			var status = handleException().getStatus();

			assertThat(status.getDescription()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveIdInDescription() {
			var status = handleException().getStatus();

			assertThat(status.getDescription()).contains(exceptionId);
		}

		@Test
		void shouldHaveExceptionIdInMetadata() {
			var metaData = handleException().getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exceptionId);
		}

		private StatusException handleException() {
			return handler.handleMissingRequiredGrpcArgumentException(exception);
		}
	}

	private Key<String> createExceptionIdKey() {
		return createKeyOf(GrpcExceptionHandlers.KEY_EXCEPTION_ID);
	}

	private Key<String> createKeyOf(String key) {
		return Key.of(key, Metadata.ASCII_STRING_MARSHALLER);
	}
}