/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.fachstelle;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelle;
import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelleGetResponse;
import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelleSearchResponse;
import io.grpc.stub.StreamObserver;

class FachstelleGrpcServiceTest {

	@Spy
	@InjectMocks
	private FachstelleGrpcService grpcService;
	@Mock
	private FachstelleSearchService searchService;
	@Mock
	private FachstelleMapper mapper;

	private final Fachstelle fachstelle = FachstelleTestFactory.create();
	private final Fachstelle fachstelle2 = FachstelleTestFactory.create();
	private final List<Fachstelle> fachstellen = List.of(fachstelle, fachstelle2);

	private final GrpcFachstelle grpcFachstelle = GrpcFachstelleTestFactory.createBuilder().setMukId("id-1").build();
	private final GrpcFachstelle grpcFachstelle2 = GrpcFachstelleTestFactory.createBuilder().setMukId("id-2").build();
	private final List<GrpcFachstelle> grpcFachstellen = List.of(grpcFachstelle, grpcFachstelle2);

	@Nested
	class TestSearch {
		@Mock
		private StreamObserver<GrpcFachstelleSearchResponse> responseObserver;

		private final GrpcFachstelleSearchResponse searchResponse = GrpcFachstelleSearchResponseTestFactory.createBuilder()
				.addAllFachstellen(grpcFachstellen).build();

		@BeforeEach
		void init() {
			when(searchService.search(GrpcFachstelleSearchRequestTestFactory.SEARCH_BY)).thenReturn(fachstellen);
			doReturn(searchResponse).when(grpcService).buildFachstelleSearchResponse(fachstellen);
		}

		@Test
		void shouldCallSearchService() {
			callGrpcService();

			verify(searchService).search(GrpcFachstelleSearchRequestTestFactory.SEARCH_BY);
		}

		@Test
		void shouldBuildResponse() {
			callGrpcService();

			verify(grpcService).buildFachstelleSearchResponse(fachstellen);
		}

		@Test
		void shouldCallOnNextWithResponse() {
			callGrpcService();

			verify(responseObserver).onNext(searchResponse);
		}

		@Test
		void shouldCallOnCompleted() {
			callGrpcService();

			verify(responseObserver).onCompleted();
		}

		private void callGrpcService() {
			grpcService.search(GrpcFachstelleSearchRequestTestFactory.create(), responseObserver);
		}
	}

	@Nested
	class TestBuildFachstelleSearchResponse {

		@BeforeEach
		void init() {
			when(mapper.toGrpc(fachstelle)).thenReturn(grpcFachstelle);
			when(mapper.toGrpc(fachstelle2)).thenReturn(grpcFachstelle2);
		}

		@Test
		void shouldCallMapper() {
			grpcService.buildFachstelleSearchResponse(fachstellen);

			fachstellen.forEach(fachstelle -> verify(mapper).toGrpc(fachstelle));
		}

		@Test
		void shouldAddFachstellenToResponse() {
			var response = grpcService.buildFachstelleSearchResponse(fachstellen);

			assertThat(response.getFachstellenList()).containsExactlyElementsOf(grpcFachstellen);
		}
	}

	@Nested
	class TestGetById {

		private final GrpcFachstelleGetResponse response = GrpcFachstelleGetResponseTestFactory.create();
		@Mock
		private StreamObserver<GrpcFachstelleGetResponse> responseObserver;

		@BeforeEach
		void init() {
			when(searchService.getById(FachstelleTestFactory.ID)).thenReturn(fachstelle);
			doReturn(response).when(grpcService).buildFachstelleGetResponse(fachstelle);
		}

		@Test
		void shouldCalSearchService() {
			callService();

			verify(searchService).getById(FachstelleTestFactory.ID);
		}

		@Test
		void shouldBuildResponse() {
			callService();

			verify(grpcService).buildFachstelleGetResponse(fachstelle);
		}

		@Test
		void shouldNotifyObserver() {
			callService();

			verify(responseObserver).onNext(response);
		}

		@Test
		void shouldCompleteResponse() {
			callService();

			verify(responseObserver).onCompleted();
		}

		private void callService() {
			grpcService.getById(GrpcFachstelleGetRequestTestFactory.create(), responseObserver);
		}
	}

	@Nested
	class TestBuildFachstelleGetResponse {

		@BeforeEach
		void init() {
			when(mapper.toGrpc(fachstelle)).thenReturn(grpcFachstelle);
		}

		@Test
		void shouldMapFachstelle() {
			callService();

			verify(mapper).toGrpc(fachstelle);
		}

		@Test
		void shouldReturnResponse() {
			var result = callService();

			assertThat(result.getFachstelle()).isEqualTo(grpcFachstelle);
		}

		private GrpcFachstelleGetResponse callService() {
			return grpcService.buildFachstelleGetResponse(fachstelle);
		}
	}
}
