/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.fachstelle;

import static de.ozgcloud.zufi.common.mongodb.DocumentTestUtils.*;
import static org.assertj.core.api.Assertions.*;
import static org.awaitility.Awaitility.*;

import java.util.concurrent.TimeUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.mongodb.core.MongoOperations;

import com.google.rpc.Code;

import de.ozgcloud.common.test.DataITCase;
import de.ozgcloud.zufi.grpc.fachstelle.FachstelleRegistrationServiceGrpc.FachstelleRegistrationServiceBlockingStub;
import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelleRegistrationRequest;
import io.grpc.StatusRuntimeException;
import net.devh.boot.grpc.client.inject.GrpcClient;

@SpringBootTest(properties = {
		"grpc.server.inProcessName=FachstelleRegistrationGrpcService-test",
		"grpc.client.inProcess.address=in-process:FachstelleRegistrationGrpcService-test"
})
@DataITCase
class FachstelleRegistrationGrpcServiceITCase {

	@GrpcClient("inProcess")
	private FachstelleRegistrationServiceBlockingStub serviceStub;

	@Autowired
	private MongoOperations mongoOperations;

	@BeforeEach
	void clearDb() {
		deleteAllFromCollection(Fachstelle.class);
	}

	@Nested
	class TestRegister {

		private final GrpcFachstelleRegistrationRequest validRequest = GrpcFachstelleRegistrationRequestTestFactory.create();
		private final GrpcFachstelleRegistrationRequest invalidRequestMissingMukId = GrpcFachstelleRegistrationRequestTestFactory.createWithoutMukId();

		@Test
		void shouldPersistFachstelle() {
			serviceStub.register(validRequest);

			await().atMost(5, TimeUnit.SECONDS).untilAsserted(() -> {
				var registeredFachstellen = mongoOperations.findAll(Fachstelle.class);
				assertThat(registeredFachstellen).hasSize(1).first().usingRecursiveComparison(COMPARE_ALL_FIELDS_BUT_ID)
						.isEqualTo(FachstelleTestFactory.createWithoutId());
			});
		}

		@Test
		void shouldThrowExceptionOnMissingMukId() {
			assertThatThrownBy(() -> serviceStub.register(invalidRequestMissingMukId)).isInstanceOf(StatusRuntimeException.class).hasMessageContainingAll(
					Code.forNumber(Code.INVALID_ARGUMENT_VALUE).toString(), "mukId");
		}
	}
}
