/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.fachstelle;

import static de.ozgcloud.zufi.common.mongodb.DocumentTestUtils.*;
import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoOperations;

import de.ozgcloud.common.test.DataITCase;
import jakarta.validation.ConstraintViolationException;

@DataITCase
class FachstelleRegistrationServiceITCase {

	@Autowired
	private FachstelleRegistrationService fachstelleRegistrationService;
	@Autowired
	private MongoOperations mongoOperations;

	@BeforeEach
	void clearDb() {
		deleteAllFromCollection(Fachstelle.class);
	}

	@Nested
	class TestRegister {

		private final Fachstelle fachstelleToRegister = FachstelleTestFactory.createWithoutId();

		@ParameterizedTest
		@NullAndEmptySource
		void shouldValidateFachstelle(String mukId) {
			var fachstelleWithoutMukId = FachstelleTestFactory.createBuilder().mukId(mukId).build();

			assertThatThrownBy(() -> fachstelleRegistrationService.register(fachstelleWithoutMukId)).isInstanceOf(ConstraintViolationException.class)
					.hasMessageContainingAll("mukId", "blank");
		}

		@Test
		void shouldSaveFachstelle() {
			fachstelleRegistrationService.register(fachstelleToRegister);

			assertThat(persistedFachstellen()).hasSize(1).first().usingRecursiveComparison(COMPARE_ALL_FIELDS_BUT_ID).isEqualTo(FachstelleTestFactory.createWithoutId());
		}

		@Test
		void shouldSetId() {
			fachstelleRegistrationService.register(fachstelleToRegister);

			assertThat(persistedFachstellen()).first().extracting(Fachstelle::getId).isNotNull();
		}

		@Nested
		class OnDocumentWithMukIdAlreadyExists {

			private static final String FIRMENNAME_2 = FachstelleTestFactory.FIRMEN_NAME + " - Neu";
			private final Fachstelle fachstelleToRegister2 = fachstelleToRegister.toBuilder().firmenName(FIRMENNAME_2).build();

			@BeforeEach
			void initDb() {
				fachstelleRegistrationService.register(fachstelleToRegister);
			}

			@Test
			void shouldNotCreateNewFachstelle() {
				fachstelleRegistrationService.register(fachstelleToRegister2);

				assertThat(persistedFachstellen()).hasSize(1);
			}

			@Test
			void shouldUpdateExistingFachstelle() {
				fachstelleRegistrationService.register(fachstelleToRegister2);

				assertThat(persistedFachstellen()).first().extracting(Fachstelle::getFirmenName).isEqualTo(FIRMENNAME_2);
			}
		}
	}

	private List<Fachstelle> persistedFachstellen() {
		return mongoOperations.findAll(Fachstelle.class);
	}
}
