/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.fachstelle;

import static de.ozgcloud.zufi.fachstelle.FachstelleTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

class FachstelleRegistrationServiceTest {
	@Spy
	@InjectMocks
	private FachstelleRegistrationService fachstelleRegistrationService;
	@Mock
	private FachstelleRepository fachstelleRepository;

	@Nested
	class TestRegister {

		private final Fachstelle fachstelleToRegister = FachstelleTestFactory.createWithoutId();

		@Test
		void shouldFindExistingFachstelle() {
			fachstelleRegistrationService.register(fachstelleToRegister);

			verify(fachstelleRegistrationService).findExistingFachstelle(MUK_ID);
		}

		@Nested
		class OnFachstelleWithMukIdAlreadyExists {

			private final Fachstelle existingFachstelle = FachstelleTestFactory.createBuilder()
					.id(ID)
					.firmenName(FIRMEN_NAME + "-old")
					.rechtsform(RECHTSFORM + "-old")
					.rechtsformText(RECHTSFORM_TEXT + "-old")
					.registerNummer(REGISTER_NUMMER + "-old")
					.registerArt(REGISTER_ART + "-old")
					.emailAdresse(EMAIL_ADRESSE + "-old")
					.anschrift(ANSCHRIFT + "-old")
					.build();

			@Captor
			private ArgumentCaptor<Fachstelle> fachstelleArgumentCaptor;

			@BeforeEach
			void init() {
				doReturn(Optional.of(existingFachstelle)).when(fachstelleRegistrationService).findExistingFachstelle(MUK_ID);
			}

			@Test
			void shouldSaveFachstelleWithIdOfExistingOne() {
				fachstelleRegistrationService.register(fachstelleToRegister);

				verify(fachstelleRepository).save(fachstelleArgumentCaptor.capture());
				assertThat(fachstelleArgumentCaptor.getValue()).usingRecursiveComparison().isEqualTo(FachstelleTestFactory.create());
			}
		}

		@Nested
		class OnFachstelleWithMukIdDoesNotYetExists {

			@BeforeEach
			void init() {
				doReturn(Optional.empty()).when(fachstelleRepository).findByMukId(MUK_ID);
			}

			@Test
			void shouldSaveFachstelle() {
				fachstelleRegistrationService.register(fachstelleToRegister);

				verify(fachstelleRepository).save(fachstelleToRegister);
			}
		}
	}

	@Nested
	class TestFindExistingFachstelle {

		@Test
		void shouldFindByMukId() {
			fachstelleRegistrationService.findExistingFachstelle(MUK_ID);

			verify(fachstelleRepository).findByMukId(MUK_ID);
		}

		@Test
		void shouldReturnFachstelle() {
			var fachstelle = FachstelleTestFactory.createWithoutId();
			when(fachstelleRepository.findByMukId(MUK_ID)).thenReturn(Optional.of(fachstelle));

			var result = fachstelleRegistrationService.findExistingFachstelle(MUK_ID);

			assertThat(result).hasValue(fachstelle);
		}
	}
}
