/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.fachstelle;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class FachstelleSearchServiceTest {

	@InjectMocks
	@Spy
	private FachstelleSearchService service;

	@Mock
	private FachstelleRepository repository;

	@Nested
	class TestGetById {

		@Nested
		class OnFachstelleFound {

			private final Fachstelle fachstelle = FachstelleTestFactory.create();

			@BeforeEach
			void init() {
				when(repository.findById(FachstelleTestFactory.ID)).thenReturn(Optional.of(fachstelle));
			}

			@Test
			void shouldFindById() {
				callService();

				verify(repository).findById(FachstelleTestFactory.ID);
			}

			@Test
			void shouldReturnFachstelle() {
				var foundFachstelle = callService();

				assertThat(foundFachstelle).isEqualTo(fachstelle);
			}
		}

		@Nested
		class OnFachstelleNotFound {

			@BeforeEach
			void init() {
				when(repository.findById(FachstelleTestFactory.ID)).thenReturn(Optional.empty());
			}

			@Test
			void shouldThrowException() {
				assertThatThrownBy(() -> callService()).isInstanceOf(FachstelleNotFoundException.class).hasMessageContaining(FachstelleTestFactory.ID);
			}
		}

		private Fachstelle callService() {
			return service.getById(FachstelleTestFactory.ID);
		}
	}

	@Nested
	class TestSearch {

		@Nested
		class OnNotBlankText {
			private final static String TEXT = LoremIpsum.getInstance().getWords(2);

			@Test
			void shouldCallRepository() {
				callService();

				verify(repository).findAllContaining(TEXT);
			}

			@Test
			void shouldReturnFoundFachstellen() {
				var expectedFachstellen = List.of(FachstelleTestFactory.create());
				when(repository.findAllContaining(TEXT)).thenReturn(expectedFachstellen);

				var fachstellen = callService();

				assertThat(fachstellen).isEqualTo(expectedFachstellen);
			}

			private List<Fachstelle> callService() {
				return service.search(TEXT);
			}
		}

		@Nested
		class OnBlankText {

			@ParameterizedTest
			@NullAndEmptySource
			@ValueSource(strings = " ")
			void shouldNotCallRepository(String text) {
				service.search(text);

				verify(repository, never()).findAllContaining(any());
			}

			@ParameterizedTest
			@NullAndEmptySource
			@ValueSource(strings = " ")
			void shouldReturnEmptyList(String text) {
				var fachstellen = service.search(text);

				assertThat(fachstellen).isEmpty();
			}
		}
	}

}
