/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcGetByOrganisationsEinheitIdResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheit;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitSearchRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitSearchResponse;
import io.grpc.stub.StreamObserver;

class OrganisationsEinheitGrpcServiceTest {
	@Spy
	@InjectMocks
	private OrganisationsEinheitGrpcService service;
	@Mock
	private OrganisationsEinheitSearchService searchService;
	@Mock
	private GrpcOrganisationsEinheitMapper mapper;

	private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();
	private final GrpcOrganisationsEinheit grpcOrganisationsEinheit = GrpcOrganisationsEinheitTestFactory.create();

	@Nested
	class TestSearchOrganisationseinheiten {

		@Mock
		private StreamObserver<GrpcOrganisationsEinheitSearchResponse> responseObserver;

		@Captor
		private ArgumentCaptor<GrpcOrganisationsEinheitSearchResponse> responseCaptor;

		private final GrpcOrganisationsEinheitSearchRequest request = GrpcOrganisationsEinheitSearchRequestTestFactory.create();
		private final OrganisationsEinheit organisationsEinheit2 = OrganisationsEinheitTestFactory.create();
		private final List<OrganisationsEinheit> organisationsEinheiten = List.of(organisationsEinheit, organisationsEinheit2);
		private final GrpcOrganisationsEinheit grpcOrganisationsEinheit2 = GrpcOrganisationsEinheitTestFactory.create();

		@Nested
		class TestSearch {

			@BeforeEach
			void setUpMocks() {
				when(searchService.search(request.getSearchBy(), request.getArs())).thenReturn(organisationsEinheiten);
				when(mapper.fromOrganisationsEinheit(organisationsEinheit)).thenReturn(grpcOrganisationsEinheit);
				when(mapper.fromOrganisationsEinheit(organisationsEinheit2)).thenReturn(grpcOrganisationsEinheit2);
			}

			@Test
			void shouldCallService() {
				callService();

				verify(searchService).search(request.getSearchBy(), request.getArs());
			}

			@Test
			void shouldBuildResponse() {
				callService();

				verify(service).buildOrganisationsEinheitSearchResponse(organisationsEinheiten);
			}

			@Test
			void shouldSetMappedOrganisationsEinheiten() {
				callService();

				verify(responseObserver).onNext(responseCaptor.capture());
				assertThat(responseCaptor.getValue().getOrganisationsEinheitenList()).containsExactly(grpcOrganisationsEinheit,
						grpcOrganisationsEinheit2);
			}

			@Test
			void shouldCompleteResponse() {
				callService();

				verify(responseObserver).onCompleted();
			}
		}

		@Nested
		class TestBuildOrganisationsEinheitSearchResponse {

			@BeforeEach
			void setUpMocks() {
				when(searchService.search(request.getSearchBy(), request.getArs())).thenReturn(organisationsEinheiten);
				when(mapper.fromOrganisationsEinheit(organisationsEinheit)).thenReturn(grpcOrganisationsEinheit);
				when(mapper.fromOrganisationsEinheit(organisationsEinheit2)).thenReturn(grpcOrganisationsEinheit2);
			}

			@Test
			void shouldSetMappedOrganisationsEinheiten() {
				callService();

				verify(responseObserver).onNext(responseCaptor.capture());
				assertThat(responseCaptor.getValue().getOrganisationsEinheitenList()).containsExactly(grpcOrganisationsEinheit,
						grpcOrganisationsEinheit2);
			}

			@Test
			void shouldCompleteResponse() {
				callService();

				verify(responseObserver).onCompleted();
			}

		}

		private void callService() {
			service.search(request, responseObserver);
		}
	}

	@Nested
	class TestGetOrganisationsEinheitById {

		private final String organisationsEinheitId = organisationsEinheit.getId();

		private final GrpcOrganisationsEinheitGetRequest request = GrpcOrganisationsEinheitGetRequestTestFactory.create();

		@Mock
		private StreamObserver<GrpcOrganisationsEinheitGetResponse> responseObserver;

		@Captor
		private ArgumentCaptor<GrpcOrganisationsEinheitGetResponse> responseCaptor;

		@Nested
		class TestGetById {

			@BeforeEach
			void setUpMocks() {
				when(searchService.getById(organisationsEinheitId)).thenReturn(organisationsEinheit);
				when(mapper.fromOrganisationsEinheit(organisationsEinheit)).thenReturn(grpcOrganisationsEinheit);
			}

			@Test
			void shouldCallService() {
				callService();

				verify(searchService).getById(organisationsEinheitId);
			}

			@Test
			void shouldBuildResponse() {
				callService();

				verify(service).buildOrganisationsEinheitGetResponse(organisationsEinheit);
			}

			@Test
			void shouldReturnOrganisationsEinheit() {
				callService();

				verify(responseObserver).onNext(responseCaptor.capture());
				assertThat(responseCaptor.getValue().getOrganisationsEinheit()).isEqualTo(grpcOrganisationsEinheit);
			}

			@Test
			void shouldCompleteResponse() {
				callService();

				verify(responseObserver).onCompleted();
			}

			private void callService() {
				service.getById(request, responseObserver);
			}
		}

		@Nested
		class TestBuildOrganisationsEinheitGetResponse {

			@BeforeEach
			void setup() {
				when(mapper.fromOrganisationsEinheit(organisationsEinheit)).thenReturn(grpcOrganisationsEinheit);
			}

			@Test
			void shouldMapOrganisationsEinheit() {
				callService();

				verify(mapper).fromOrganisationsEinheit(organisationsEinheit);
			}

			@Test
			void shouldReturnResponse() {
				var result = callService();

				assertThat(result.getOrganisationsEinheit()).isEqualTo(grpcOrganisationsEinheit);
			}

			private GrpcOrganisationsEinheitGetResponse callService() {
				return service.buildOrganisationsEinheitGetResponse(organisationsEinheit);
			}
		}

	}

	@Nested
	class TestGetByOrganisationsEinheitId {

		public static final String ORGANISATIONS_EINHEIT_ID = OrganisationsEinheitTestFactory.XZUFI_ID.getId();

		private final List<OrganisationsEinheit> organisationsEinheiten = List.of(organisationsEinheit);
		private final GrpcGetByOrganisationsEinheitIdResponse response = GrpcGetByOrganisationsEinheitIdResponseTestFactory.create();

		@Mock
		private StreamObserver<GrpcGetByOrganisationsEinheitIdResponse> responseObserver;

		@Captor
		private ArgumentCaptor<GrpcGetByOrganisationsEinheitIdResponse> responseCaptor;

		@BeforeEach
		void init() {
			when(searchService.getByOrganisationsEinheitId(ORGANISATIONS_EINHEIT_ID)).thenReturn(organisationsEinheiten);
			doReturn(response).when(service).buildGetByOrganisationsEinheitIdResponse(organisationsEinheiten);
		}

		@Test
		void shouldCallService() {
			callService();

			verify(searchService).getByOrganisationsEinheitId(ORGANISATIONS_EINHEIT_ID);
		}

		@Test
		void shouldBuildResponse() {
			callService();

			verify(service).buildGetByOrganisationsEinheitIdResponse(organisationsEinheiten);
		}

		@Test
		void shouldReturnResponse() {
			callService();

			verify(responseObserver).onNext(responseCaptor.capture());
			assertThat(responseCaptor.getValue()).isEqualTo(response);
		}

		@Test
		void shouldCompleteResponse() {
			callService();

			verify(responseObserver).onCompleted();
		}

		private void callService() {
			service.getByOrganisationsEinheitId(GrpcGetByOrganisationsEinheitIdRequestTestFactory.create(), responseObserver);
		}
	}

	@Nested
	class TestBuildGetByOrganisationsEinheitIdResponse {

		private final List<OrganisationsEinheit> organisationsEinheiten = List.of(organisationsEinheit);

		@BeforeEach
		void setup() {
			when(mapper.fromOrganisationsEinheit(organisationsEinheit)).thenReturn(grpcOrganisationsEinheit);
		}

		@Test
		void shouldMapOrganisationsEinheit() {
			callService();

			verify(mapper).fromOrganisationsEinheit(organisationsEinheit);
		}

		@Test
		void shouldReturnResponse() {
			var result = callService();

			assertThat(result.getOrganisationsEinheitenList()).containsExactly(GrpcGetByOrganisationsEinheitIdResponseTestFactory.GRPC_ORGANISATIONS_EINHEIT);
		}

		private GrpcGetByOrganisationsEinheitIdResponse callService() {
			return service.buildGetByOrganisationsEinheitIdResponse(organisationsEinheiten);
		}
	}
}
