/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class OrganisationsEinheitSearchServiceTest {
	@InjectMocks
	@Spy
	private OrganisationsEinheitSearchService service;

	@Mock
	private OrganisationsEinheitRepository repository;

	@Nested
	class TestGetById {

		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

		@Nested
		class WhenHavingOrganisationsEinheit {
			@BeforeEach
			void setup() {
				when(repository.findById(OrganisationsEinheitTestFactory.ID)).thenReturn(Optional.of(organisationsEinheit));
			}

			@Test
			void shouldFindById() {
				callService();

				verify(repository).findById(OrganisationsEinheitTestFactory.ID);
			}

			@Test
			void shouldReturnOrganisationsEinheit() {
				var result = callService();

				assertThat(result).isEqualTo(organisationsEinheit);
			}
		}

		@Test
		void shouldThrowExceptionIfNotFound() {
			assertThrows(OrganisationsEinheitNotFoundException.class, () -> callService());
		}

		private OrganisationsEinheit callService() {
			return service.getById(OrganisationsEinheitTestFactory.ID);
		}
	}

	@Nested
	class TestGetByOrganisationsEinheitId {

		public static final String ORGANISATIONS_EINHEIT_ID = OrganisationsEinheitTestFactory.XZUFI_ID.getId();

		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void init() {
			when(repository.findByOrganisationsEinheitId(ORGANISATIONS_EINHEIT_ID)).thenReturn(List.of(organisationsEinheit));
		}

		@Test
		void shouldFindByOrganisationsEinheitId() {
			callService();

			verify(repository).findByOrganisationsEinheitId(ORGANISATIONS_EINHEIT_ID);
		}

		@Test
		void shouldReturnOrganisationsEinheiten() {
			var result = callService();

			assertThat(result).containsExactly(organisationsEinheit);
		}

		private List<OrganisationsEinheit> callService() {
			return service.getByOrganisationsEinheitId(ORGANISATIONS_EINHEIT_ID);
		}
	}

	@Nested
	class TestSearch {

		private final String text = LoremIpsum.getInstance().getWords(5);
		private final String ars = UUID.randomUUID().toString();

		@Nested
		class OnEmptyArs {

			@ParameterizedTest
			@NullAndEmptySource
			void shouldCallRepository(String ars) {
				callService(ars);

				verify(repository).findAllContaining(text);
			}

			@ParameterizedTest
			@NullAndEmptySource
			void shouldReturnFoundOrganisationsEinheiten(String ars) {
				var organisationsEinheiten = List.of(OrganisationsEinheitTestFactory.create());
				when(repository.findAllContaining(text)).thenReturn(organisationsEinheiten);

				var found = callService(ars);

				assertThat(found).isEqualTo(organisationsEinheiten);
			}
		}

		@Nested
		class OnNonEmptyArs {

			@Test
			void shouldCallRepository() {
				callService();

				verify(repository).findAllContainingInGebiet(text, ars);
			}

			@Test
			void shouldReturnFoundOrganisationsEinheiten() {
				var organisationsEinheiten = List.of(OrganisationsEinheitTestFactory.create());
				when(repository.findAllContainingInGebiet(text, ars)).thenReturn(organisationsEinheiten);

				var found = callService();

				assertThat(found).isEqualTo(organisationsEinheiten);
			}
		}

		private List<OrganisationsEinheit> callService() {
			return callService(ars);
		}

		private List<OrganisationsEinheit> callService(String ars) {
			return service.search(text, ars);
		}
	}
}
