/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static de.ozgcloud.zufi.organisationseinheit.GrpcVorgangManagerRegistrationRequestTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.zufi.common.errorhandling.MissingRequiredGrpcArgumentException;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerGetAddressRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerGetAddressResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerRegistrationRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcVorgangManagerRegistrationResponse;
import io.grpc.stub.StreamObserver;

class VorgangManagerRegistrationGrpcServiceTest {
	@Spy
	@InjectMocks
	private VorgangManagerRegistrationGrpcService grpcService;

	@Mock
	private VorgangManagerRegistrationService service;

	@Mock
	private GrpcXzufiIdMapper xzufiIdMapper;

	@Nested
	class TestRegister {
		@Mock
		private StreamObserver<GrpcVorgangManagerRegistrationResponse> responseObserver;

		private final GrpcVorgangManagerRegistrationRequest request = GrpcVorgangManagerRegistrationRequestTestFactory.create();

		@BeforeEach
		void mockGrpcService() {
			doReturn(ORGANISATIONS_EINHEITEN_IDS).when(grpcService).registerAndReturnNotFoundIds(request);
		}

		@Test
		void shouldValidateServiceAddress() {
			callService();

			verify(grpcService).validateServiceAddress(request);
		}

		@Test
		void shouldRegisterAndReturnNotFoundIds() {
			callService();

			verify(grpcService).registerAndReturnNotFoundIds(request);
		}

		@Test
		void shouldBuildResponseWithNotFoundOrganisationsEinheitIds() {
			callService();

			verify(grpcService).buildRegisterResponse(ORGANISATIONS_EINHEITEN_IDS);
		}

		@Test
		void shouldSetResponse() {
			var response = GrpcVorgangManagerRegistrationResponseTestFactory.create();
			doReturn(response).when(grpcService).buildRegisterResponse(ORGANISATIONS_EINHEITEN_IDS);

			callService();

			verify(responseObserver).onNext(response);
		}

		@Test
		void shouldComplete() {
			callService();

			verify(responseObserver).onCompleted();
		}

		private void callService() {
			grpcService.register(request, responseObserver);
		}

	}

	@Nested
	class TestValidateServiceAddress {

		@Test
		void shouldThrowMissingRequiredGrpcArgumentExceptionOnMissingVorgangManagerAddress() {
			var requestWithMissingVorgangmanagerAddress = GrpcVorgangManagerRegistrationRequestTestFactory.createBuilder()
					.setServiceAddress(StringUtils.EMPTY)
					.build();

			assertThrows(MissingRequiredGrpcArgumentException.class,
					() -> callService(requestWithMissingVorgangmanagerAddress));
		}

		@Test
		void shouldNotThrowException() {
			var validRequest = GrpcVorgangManagerRegistrationRequestTestFactory.create();

			assertDoesNotThrow(() -> callService(validRequest));
		}

		private void callService(GrpcVorgangManagerRegistrationRequest request) {
			grpcService.validateServiceAddress(request);
		}
	}

	@Nested
	class TestRegisterAndReturnNotFoundIds {

		@Nested
		class OnNonEmptyXzufiIds {
			private final GrpcVorgangManagerRegistrationRequest request = GrpcVorgangManagerRegistrationRequestTestFactory.create();

			@Test
			void shouldRegisterWithXzufiIds() {
				grpcService.registerAndReturnNotFoundIds(request);

				verify(grpcService).registerWithXzufiIds(request);
			}

			@Test
			void shouldNotRegisterWithOrganisationsEinheitIds() {
				grpcService.registerAndReturnNotFoundIds(request);

				verify(grpcService, never()).registerWithOrganisationsEinheitIds(any());
			}

			@Test
			void shouldReturnNotFoundOrganisationsEinheitIds() {
				doReturn(List.of(GrpcXzufiIdTestFactory.ORGANISATIONS_EINHEIT_ID)).when(grpcService)
						.registerWithXzufiIds(request);

				var notFoundOrganisationsEinheiten = grpcService.registerAndReturnNotFoundIds(request);

				assertThat(notFoundOrganisationsEinheiten).containsExactly(GrpcXzufiIdTestFactory.ORGANISATIONS_EINHEIT_ID);
			}
		}

		@Nested
		class OnEmptyXzufiIds {
			private final GrpcVorgangManagerRegistrationRequest request = GrpcVorgangManagerRegistrationRequestTestFactory.createBuilder()
					.clearXzufiIds().build();

			@Test
			void shouldNotRegisterWithXzufiIds() {
				grpcService.registerAndReturnNotFoundIds(request);

				verify(grpcService, never()).registerWithXzufiIds(any());
			}

			@Test
			void shouldRegisterWithOrganisationsEinheitIds() {
				grpcService.registerAndReturnNotFoundIds(request);

				verify(grpcService).registerWithOrganisationsEinheitIds(request);
			}

			@Test
			void shouldReturnNotFoundOrganisationsEinheitIds() {
				doReturn(List.of(GrpcXzufiIdTestFactory.ORGANISATIONS_EINHEIT_ID)).when(grpcService)
						.registerWithOrganisationsEinheitIds(request);

				var notFoundOrganisationsEinheiten = grpcService.registerAndReturnNotFoundIds(request);

				assertThat(notFoundOrganisationsEinheiten).containsExactly(GrpcXzufiIdTestFactory.ORGANISATIONS_EINHEIT_ID);
			}
		}

		@Nested
		class OnEmptyXzufiIdsAndOrganisationsEinheitIds {
			private final GrpcVorgangManagerRegistrationRequest request = GrpcVorgangManagerRegistrationRequestTestFactory.createBuilder()
					.clearXzufiIds()
					.clearOrganisationsEinheitIds()
					.build();

			@Test
			void shouldThrowMissingRequiredGrpcArgumentExcpetionOnMissingOrganisationsEinheitIds() {
				assertThrows(MissingRequiredGrpcArgumentException.class,
						() -> grpcService.registerAndReturnNotFoundIds(request));
			}
		}

	}

	@Nested
	class TestRegisterWithXzufiIds {
		private final GrpcVorgangManagerRegistrationRequest request = GrpcVorgangManagerRegistrationRequestTestFactory.create();

		private final XzufiId xzufiId1 = XzufiIdTestFactory.create();
		private final XzufiId xzufiId2 = XzufiIdTestFactory.createBuilder()
				.id(UUID.randomUUID().toString())
				.schemeAgencyId(UUID.randomUUID().toString())
				.build();

		@BeforeEach
		void mockMapper() {
			when(xzufiIdMapper.fromGrpc(GRPC_ORGANISATIONS_EINHEIT_XZUFI_ID_1)).thenReturn(xzufiId1);
			when(xzufiIdMapper.fromGrpc(GRPC_ORGANISATIONS_EINHEIT_XZUFI_ID_2)).thenReturn(xzufiId2);
		}

		@Test
		void shouldMapXzufiIds() {
			callService();

			GRPC_ORGANISATIONS_EINHEIT_XZUFI_IDS.forEach(id -> verify(xzufiIdMapper).fromGrpc(id));
		}

		@Test
		void shouldCallService() {
			callService();

			List.of(xzufiId1, xzufiId2).forEach(id -> verify(service).registerVorgangManager(ADDRESS, id));
		}

		@Nested
		class OnAllFoundOrganisationsEinheit {

			@BeforeEach
			void mockService() {
				List.of(xzufiId1, xzufiId2)
						.forEach(id -> when(service.registerVorgangManager(ADDRESS, id)).thenReturn(true));
			}

			@Test
			void shouldReturnEmptyList() {
				var notFoundOrganisationsEinheiten = callService();

				assertThat(notFoundOrganisationsEinheiten).isEmpty();
			}

		}

		@Nested
		class OnPartialNotFoundOrganisationsEinheit {

			@BeforeEach
			void mockService() {
				when(service.registerVorgangManager(ADDRESS, xzufiId1)).thenReturn(false);
				when(service.registerVorgangManager(ADDRESS, xzufiId2)).thenReturn(true);
			}

			@Test
			void shouldBuildResponseWithMissingOrganisationsEinheitIds() {
				var notFoundOrganisationsEinheiten = callService();

				assertThat(notFoundOrganisationsEinheiten).containsExactly(GrpcXzufiIdTestFactory.SCHEME_AGENCY_ID + "."
						+ GrpcXzufiIdTestFactory.ORGANISATIONS_EINHEIT_ID);
			}

		}

		private List<String> callService() {
			return grpcService.registerWithXzufiIds(request);
		}
	}

	@Nested
	class TestRegisterWithOrganisationsEinheitIds {
		private final GrpcVorgangManagerRegistrationRequest request = GrpcVorgangManagerRegistrationRequestTestFactory.create();

		@Test
		void shouldCallService() {
			callService();

			ORGANISATIONS_EINHEITEN_IDS.forEach(id -> verify(service).registerVorgangManagerWithOrganisationsEinheit(ADDRESS, id));
		}

		@Nested
		class OnAllFoundOrganisationsEinheit {

			@BeforeEach
			void mockService() {
				ORGANISATIONS_EINHEITEN_IDS
						.forEach(id -> when(service.registerVorgangManagerWithOrganisationsEinheit(ADDRESS, id)).thenReturn(true));
			}

			@Test
			void shouldReturnEmptyList() {
				var notFoundOrganisationsEinheiten = callService();

				assertThat(notFoundOrganisationsEinheiten).isEmpty();
			}

		}

		@Nested
		class OnPartialNotFoundOrganisationsEinheit {

			@BeforeEach
			void mockService() {
				when(service.registerVorgangManagerWithOrganisationsEinheit(ADDRESS, ORGANISATIONS_EINHEITEN_IDS.get(0))).thenReturn(false);
				when(service.registerVorgangManagerWithOrganisationsEinheit(ADDRESS, ORGANISATIONS_EINHEITEN_IDS.get(1))).thenReturn(true);
			}

			@Test
			void shouldBuildResponseWithMissingOrganisationsEinheitIds() {
				var notFoundOrganisationsEinheiten = callService();

				assertThat(notFoundOrganisationsEinheiten).containsExactly(ORGANISATIONS_EINHEITEN_IDS.get(0));
			}

		}

		private List<String> callService() {
			return grpcService.registerWithOrganisationsEinheitIds(request);
		}
	}

	@Nested
	class TestBuildRegisterResponse {
		@Nested
		class OnEmptyMissingOrganisationsEinheitIds {

			@Test
			void shouldHaveSuccessfulTrue() {
				var response = callService();

				assertThat(response.getSuccess()).isTrue();
			}

			@Test
			void shouldHaveEmptyMessage() {
				var response = callService();

				assertThat(response.getMessage()).isEmpty();
			}

			GrpcVorgangManagerRegistrationResponse callService() {
				return grpcService.buildRegisterResponse(Collections.emptyList());
			}
		}

		@Nested
		class OnFilledMissingOrganisationsEinheitIds {

			@Test
			void shouldHaveSuccessfulFalse() {
				var response = callService();

				assertThat(response.getSuccess()).isFalse();
			}

			@Test
			void shouldSetMessage() {
				var expectedMessage = "Could not find the following OrganisationsEinheitIds: "
						+ ORGANISATIONS_EINHEITEN_IDS.get(0) + ", "
						+ ORGANISATIONS_EINHEITEN_IDS.get(1);

				var response = callService();

				assertThat(response.getMessage()).isEqualTo(expectedMessage);
			}

			GrpcVorgangManagerRegistrationResponse callService() {
				return grpcService.buildRegisterResponse(ORGANISATIONS_EINHEITEN_IDS);
			}
		}
	}

	@Nested
	class TestGetVorgangManagerAddress {

		private static final String ORGANISATIONSEINHEIT_ID = GrpcVorgangManagerGetAddressRequestTestFactory.ORGANISATIONSEINHEIT_ID;
		private static final String VORGANG_MANAGER_ADDRESS = GrpcVorgangManagerGetAddressResponseTestFactory.VORGANG_MANAGER_ADDRESS;
		private final GrpcVorgangManagerGetAddressRequest request = GrpcVorgangManagerGetAddressRequestTestFactory.create();

		@Mock
		private StreamObserver<GrpcVorgangManagerGetAddressResponse> responseObserver;
		@Captor
		private ArgumentCaptor<GrpcVorgangManagerGetAddressResponse> responseCaptor;

		@Test
		void shouldCallService() {
			when(service.getVorgangManagerAddress(ORGANISATIONSEINHEIT_ID)).thenReturn(Optional.of(LoremIpsum.getInstance().getWords(1)));

			callService();

			verify(service).getVorgangManagerAddress(ORGANISATIONSEINHEIT_ID);
		}

		@Nested
		class OnFoundOrganisationsEinheit {

			@Test
			void shouldSetAddress() {
				when(service.getVorgangManagerAddress(ORGANISATIONSEINHEIT_ID)).thenReturn(Optional.of(VORGANG_MANAGER_ADDRESS));

				callService();

				verify(responseObserver).onNext(responseCaptor.capture());
				assertThat(responseCaptor.getValue().getVorgangManagerAddress()).isEqualTo(VORGANG_MANAGER_ADDRESS);
			}

			@Test
			void shouldComplete() {
				when(service.getVorgangManagerAddress(ORGANISATIONSEINHEIT_ID)).thenReturn(Optional.of(VORGANG_MANAGER_ADDRESS));

				callService();

				verify(responseObserver).onCompleted();
			}

		}

		@Nested
		class OnUnknownOrganisationsEinheitId {
			@BeforeEach
			void mockService() {
				when(service.getVorgangManagerAddress(ORGANISATIONSEINHEIT_ID)).thenReturn(Optional.empty());
			}

			@Test
			void shouldThrowException() {
				assertThrows(OrganisationsEinheitNotFoundException.class, () -> callService());
			}

		}

		private void callService() {
			grpcService.getVorgangManagerAddress(request, responseObserver);
		}
	}

}
