/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class VorgangManagerRegistrationServiceTest {

	@Spy
	@InjectMocks
	private VorgangManagerRegistrationService service;

	@Mock
	private OrganisationsEinheitRepository repository;

	private static final String ORGANISATIONS_EINHEIT_ID = XzufiIdTestFactory.ID;
	private static final String VORGANG_MANAGER_ADDRESS = OrganisationsEinheitTestFactory.VORGANG_MANAGER_ADDRESS;

	@Nested
	class TestRegisterVorgangManager {
		private final Optional<OrganisationsEinheit> organisationsEinheit = Optional.of(OrganisationsEinheitTestFactory.create());

		@BeforeEach
		void mockRepository() {
			when(repository.findFirstByXzufiId(OrganisationsEinheitTestFactory.XZUFI_ID))
					.thenReturn(organisationsEinheit);
		}

		@Test
		void shouldCallRepository() {
			callService();

			verify(repository).findFirstByXzufiId(OrganisationsEinheitTestFactory.XZUFI_ID);
		}

		@Test
		void shouldRegisterWithVorgangManagerAddress() {
			callService();

			verify(service).register(organisationsEinheit, VORGANG_MANAGER_ADDRESS);
		}

		@ParameterizedTest
		@ValueSource(booleans = { false, true })
		void shouldReturnHasRegistered(boolean hasRegistered) {
			doReturn(hasRegistered).when(service).register(organisationsEinheit, VORGANG_MANAGER_ADDRESS);

			var registered = callService();

			assertThat(registered).isEqualTo(hasRegistered);
		}

		private boolean callService() {
			return service.registerVorgangManager(VORGANG_MANAGER_ADDRESS, XzufiIdTestFactory.create());
		}
	}

	@Nested
	class TestRegisterVorgangManagerWithOrganisationsEinheit {
		private final Optional<OrganisationsEinheit> organisationsEinheit = Optional.of(OrganisationsEinheitTestFactory.create());

		@BeforeEach
		void mockRepository() {
			when(repository.findFirstByXzufiIdId(ORGANISATIONS_EINHEIT_ID)).thenReturn(organisationsEinheit);
		}

		@Test
		void shouldCallRepository() {
			callService();

			verify(repository).findFirstByXzufiIdId(ORGANISATIONS_EINHEIT_ID);
		}

		@Test
		void shouldRegisterWithVorgangManagerAddress() {
			callService();

			verify(service).register(organisationsEinheit, VORGANG_MANAGER_ADDRESS);
		}

		@ParameterizedTest
		@ValueSource(booleans = { false, true })
		void shouldReturnHasRegistered(boolean hasRegistered) {
			doReturn(hasRegistered).when(service).register(any(), eq(VORGANG_MANAGER_ADDRESS));

			var registered = callService();

			assertThat(registered).isEqualTo(hasRegistered);
		}

		private boolean callService() {
			return service.registerVorgangManagerWithOrganisationsEinheit(VORGANG_MANAGER_ADDRESS, ORGANISATIONS_EINHEIT_ID);
		}
	}

	@Nested
	class TestRegister {

		@Nested
		class OnPresentOrganisationsEinheit {
			private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

			@Test
			void shouldUpdateOrganisationseinheit() {
				callService();

				verify(service).updateOrganisationsEinheit(organisationsEinheit, VORGANG_MANAGER_ADDRESS);
			}

			@Test
			void shouldReturnTrue() {
				doReturn(OrganisationsEinheitTestFactory.create())
						.when(service).updateOrganisationsEinheit(organisationsEinheit, VORGANG_MANAGER_ADDRESS);

				var hasRegistered = callService();

				assertThat(hasRegistered).isTrue();
			}

			private boolean callService() {
				return service.register(Optional.of(organisationsEinheit), VORGANG_MANAGER_ADDRESS);
			}
		}

		@Nested
		class OnMissingOrganisationsEinheit {

			@Test
			void shouldNotUpdateOrganisationseinheit() {
				callService();

				verify(service, never()).updateOrganisationsEinheit(any(), any());
			}

			@Test
			void shouldReturnFalse() {
				var hasRegistered = callService();

				assertThat(hasRegistered).isFalse();
			}

			private boolean callService() {
				return service.register(Optional.empty(), VORGANG_MANAGER_ADDRESS);
			}
		}

	}

	@Nested
	class TestUpdateOrganisationsEinheit {
		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.createBuilder().vorgangManagerAddress(null).build();
		private final OrganisationsEinheit updatedOrganisationsEinheit = OrganisationsEinheitTestFactory.create();
		private final OrganisationsEinheit savedOrganisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void setUpMock() {
			doReturn(updatedOrganisationsEinheit).when(service).setVorgangManagerAddress(organisationsEinheit, VORGANG_MANAGER_ADDRESS);
			when(repository.save(updatedOrganisationsEinheit)).thenReturn(savedOrganisationsEinheit);
		}

		@Test
		void shouldSetVorgangmanagerAddress() {
			callService();

			verify(service).setVorgangManagerAddress(organisationsEinheit, VORGANG_MANAGER_ADDRESS);

		}

		@Test
		void shouldSaveUpdatedOrganisationseinheit() {
			callService();

			verify(repository).save(updatedOrganisationsEinheit);
		}

		@Test
		void shouldReturnSavedOrganisationsEinheit() {
			var returnedOrganisationsEinheit = callService();

			assertThat(returnedOrganisationsEinheit).isEqualTo(savedOrganisationsEinheit);
		}

		private OrganisationsEinheit callService() {
			return service.updateOrganisationsEinheit(organisationsEinheit, VORGANG_MANAGER_ADDRESS);
		}
	}

	@Nested
	class TestSetVorgangManagerAddress {
		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.createBuilder().vorgangManagerAddress(null).build();

		@Test
		void shouldSetVorgangManagerAddress() {
			var resultOrganisationseinheit = service.setVorgangManagerAddress(organisationsEinheit, VORGANG_MANAGER_ADDRESS);

			assertThat(resultOrganisationseinheit).usingRecursiveComparison().isEqualTo(OrganisationsEinheitTestFactory.create());
		}
	}

	@Nested
	class TestGetVorgangManagerAddress {

		@Test
		void shouldCallRepository() {
			callService();

			verify(repository).findFirstByXzufiIdId(ORGANISATIONS_EINHEIT_ID);
		}

		@Nested
		class OnFoundOrganisationsEinheit {
			private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

			@BeforeEach
			void mockRepository() {
				when(repository.findFirstByXzufiIdId(ORGANISATIONS_EINHEIT_ID)).thenReturn(Optional.of(organisationsEinheit));
			}

			@Test
			void shouldGetVorgangManagerAddressOrDefault() {
				callService();

				verify(service).getVorgangManagerAddressOrDefault(organisationsEinheit);
			}

			@Test
			void shouldReturnVorgangManagerAddress() {
				var expectedAddress = LoremIpsum.getInstance().getUrl();
				doReturn(expectedAddress).when(service).getVorgangManagerAddressOrDefault(organisationsEinheit);

				var resultAddress = callService();

				assertThat(resultAddress).contains(expectedAddress);
			}
		}

		@Nested
		class OnOrganisationsEinheitNotInRepository {

			@BeforeEach
			void mockRepository() {
				when(repository.findFirstByXzufiIdId(ORGANISATIONS_EINHEIT_ID)).thenReturn(Optional.empty());
			}

			@Test
			void shouldReturnEmpty() {
				var searchResult = callService();

				assertThat(searchResult).isEmpty();
			}

		}

		private Optional<String> callService() {
			return service.getVorgangManagerAddress(ORGANISATIONS_EINHEIT_ID);
		}
	}

	@Nested
	class TestGetVorgangManagerAddressOrDefault {
		@Nested
		class OnOrganisationsEinheitWithoutRegisteredVorgangManager {
			@Test
			void shouldReturnEmptyString() {
				var address = service
						.getVorgangManagerAddressOrDefault(OrganisationsEinheitTestFactory.createBuilder().vorgangManagerAddress(null).build());

				assertThat(address).isEmpty();
			}
		}

		@Nested
		class OnOrganisationsEinheitWithRegisteredVorgangManager {
			@Test
			void shouldReturnVorgangManagerAddress() {
				var address = service.getVorgangManagerAddressOrDefault(OrganisationsEinheitTestFactory.create());

				assertThat(address).isEqualTo(VORGANG_MANAGER_ADDRESS);
			}
		}
	}
}
